@@ -289,7 +289,7 @@ VCustomizedMenuBarImp::menu_choosed(VWindow *vwin, USHORT id)
 		VCustomizedMenuBar::menu_lock(__FILE__,__LINE__);
 		VMenuItem *item = vwin->get_menu_bar()->search(VMT_WINDOW);
 		if  ( item == 0 )
-			er_panic("id < 0x8000 is reserved for menu");
+			er_panic("No Menu Item");
 		MENUITEMINFO mii;
 		mii.cbSize = sizeof(mii);
 		mii.fMask = MIIM_SUBMENU;
@@ -297,7 +297,7 @@ VCustomizedMenuBarImp::menu_choosed(VWindow *vwin, USHORT id)
 			err = GetLastError();
 		}
 		if ( mii.hSubMenu == 0 )
-			er_panic("Window menu must have submenu");
+			er_panic("SubMenu");
 		TCHAR str[256];
 		mii.dwTypeData = str;
 		mii.fMask = MIIM_DATA | MIIM_TYPE;
