@@ -758,6 +758,7 @@ int i;
 L_CHAR * currentPhase_mode;
 XL_SEXP * current_phase[MI_MAX];
 L_CHAR * active_phase_mode;
+char * err_msg;
 
 	cp->m = 0;
 	cp->mode = 0;
@@ -766,6 +767,7 @@ L_CHAR * active_phase_mode;
 
 	filename = get_sf_attribute(sf,l_string(std_cm,"filename"));
 /*
+	err_msg = "no filename";
 	if ( filename == 0 )
 		goto inv_param;
 */
@@ -774,6 +776,7 @@ L_CHAR * active_phase_mode;
 	currentPhase_mode = get_sf_attribute(sf,l_string(std_cm,"currentPhase_mode"));
 	active_phase_mode = get_sf_attribute(sf,l_string(std_cm,"activePhase_mode"));
 	if ( active_phase_mode ) {
+		err_msg="activePhase_mode";
 		if ( l_strcmp(active_phase_mode,l_string(std_cm,"direct")) == 0 )
 			cw.active_phase_mode = MI_MODE_DIRECT;
 		else if ( l_strcmp(active_phase_mode,l_string(std_cm,"server")) == 0 )
@@ -790,6 +793,7 @@ L_CHAR * active_phase_mode;
 		0,
 		0);
 
+	err_msg="open_matrix";
 	if ( m == 0 )
 		goto open_err;
 
@@ -863,7 +867,7 @@ L_CHAR * active_phase_mode;
 				s->h.line,
 				XLE_PROTO_UNSUTISFIED_INFO,
 				l_string(std_cm,"gmxCreate/gmxPhase"),
-				List(n_get_string("mode phase"),get_integer(er,0),-1));
+				List(n_get_string("mode phase(DIRECT save)"),get_integer(er,0),-1));
 	}
 
 	if ( cw.phase_active[MI_MODE_SERVER] ) {
@@ -873,7 +877,7 @@ L_CHAR * active_phase_mode;
 				s->h.line,
 				XLE_PROTO_UNSUTISFIED_INFO,
 				l_string(std_cm,"gmxCreate/gmxPhase"),
-				List(n_get_string("mode phase"),get_integer(er,0),-1));
+				List(n_get_string("mode phase(SERVER save)"),get_integer(er,0),-1));
 	}
 	if ( cw.phase_active[MI_MODE_CLIENT] ) {
 		if ( (er=save_mode_cal_sexp(m,MI_MODE_CLIENT,cw.phase[MI_MODE_CLIENT])) < 0 )
@@ -882,12 +886,13 @@ L_CHAR * active_phase_mode;
 				s->h.line,
 				XLE_PROTO_UNSUTISFIED_INFO,
 				l_string(std_cm,"gmxCreate/gmxPhase"),
-				List(n_get_string("mode phase"),get_integer(er,0),-1));
+				List(n_get_string("mode phase(CLIENT save)"),get_integer(er,0),-1));
 	}
 	
 	if ( currentPhase_mode == 0 )
 		goto next;
 	memset(&p,0,sizeof(p));
+	err_msg = "currentPhase_mode";
 	if ( l_strcmp(currentPhase_mode,l_string(std_cm,"direct")) == 0 )
 		er=load_mode_cal(current_phase,p.pri_area,m,MI_MODE_DIRECT);
 	else if ( l_strcmp(currentPhase_mode,l_string(std_cm,"server")) == 0 )
@@ -924,7 +929,8 @@ open_err:
 		s->h.line,
 		XLE_PROTO_OPEN_FILE,
 		l_string(std_cm,"gmxCreate"),
-		0);
+		List(n_get_string("open file error"),
+			n_get_string(err_msg),-1));
 	goto err0;
 /*
 type_missmatch:
@@ -941,7 +947,8 @@ inv_param:
 		s->h.line,
 		XLE_PROTO_INV_PARAM,
 		l_string(std_cm,"gmxCreate"),
-		0);
+		List(n_get_string("inv_param"),
+			n_get_string(err_msg),-1));
 /*
 permission_error:
 	return get_error(
