@@ -492,6 +492,9 @@ res_matrix_before.total_cnt,res_matrix_after.total_cnt);
 						-1),
 					-1);
 			d_f_ree(f);
+/*
+ss_printf("MATRIX DIMCODE=%s\n",pt_dc(t->token->process_node->matrix,t->token->process_node->dim_code,PTDC_PIXEL_1));
+*/
 			t->ret = remote_session(gblisp_top_env0,ses,
 				&r->h.target,
 				0,
@@ -620,11 +623,9 @@ ss_printf("CALL gmxCreate %ls %ls\n",filename,get_url_str2(&t->r->h.target));
 			if ( t->r->h.flags & RF_REQUIRED_PERMISSION )
 				cp.p.flags = MPF_PERMISSION;
 
-ss_printf("FLAGS-4 = %p %x %x\n",t->r,t->r->h.flags,RF_REQUIRED_PERMISSION);
 			ret = gb_gmxCreate_matrix(&cp,resource_matrix_env,cmd,
 				resource_matrix_gmxEnv,sym->symbol.field);
 
-ss_printf("FLAGS-2 = %x\n",cp.m->p.flags);
 			if ( get_type(ret) == XLT_ERROR ) {
 				print_sexp(s_stdout,ret,0);
 				ss_printf("\n");
