@@ -66,10 +66,12 @@ init_gmx_lib_function(XLISP_ENV * env0,XLISP_ENV * env1)
 	init_gmxClose(env0,env1);
 	init_gmxPNMStatus(env0,env1);
 	init_gmxImportPNM(env0,env1);
+	init_gmxImportUTM(env0,env1);
 	init_gmxExportPNM(env0,env1);
 	init_gmxScan(env0,env1);
 	init_gmxStatus(env0,env1);
 	init_gmxCopy(env0,env1);
+	init_gmxCache(env0,env1);
 
 
 	init_mxPgPlot(env1);
