@@ -390,6 +390,8 @@ change_endian_rs_res_status(PN_RS_RES_STATUS * st)
 {
 	change_endian_header(&st->pn_h);
 	change_endian_i(st->type);
+	if ( strcmp(st->version,RS_CACHE_VERSION) )
+		return -2;
 	return 0;
 }
 
@@ -806,9 +808,14 @@ RS_CONVERT_TBL * t;
 			ret = -1;
 			continue;
 		}
-		if ( (*t->change_endian_to_host)(r->data) < 0 ) {
-			ret = -1;
-			continue;
+		if ( (ret=(*t->change_endian_to_host)(r->data)) < 0 ) {
+			switch ( ret ) {
+			case -2:
+				goto loop_end;
+			case 0:
+			default:
+				continue;
+			}
 		}
 		if ( t->chain_to_host == 0 )
 			continue;
@@ -820,6 +827,7 @@ RS_CONVERT_TBL * t;
 			continue;
 		}
 	}
+loop_end:
 	return ret;
 }
 
@@ -1629,6 +1637,8 @@ L_CHAR * f;
 		goto end2;
 	get_url2(&u_target,(L_CHAR*)c->ptr);
 	ret = new_resource_entry(rh->type,&u_entry);
+	if ( ret == 0 )
+		goto end2;
 	set_resource_target(ret,&u_target);
 	ret->h.flags = ((~(RF_COMPLETE|RF_FIX))&
 				rh->flags) |
@@ -2137,6 +2147,8 @@ RS_CONVERT_TBL * t;
 		goto end;
 	}
 	ret = (*t->resource_to_host)(p,b);
+	if ( ret == 0 )
+			goto end;
 	ret->h.cache_file_id = b->file_id;
 end:
 	free_recordlist(lst);
