@@ -762,7 +762,7 @@ REAL1 * _reso;
 		pp.x = pp.x + lambda_rate;
 		*p = pp;
 	}
-	adjust_lambda(l_string(std_cm,"rad"),_p->x - M_PI,_p,_reso,0,size);
+	adjust_lambda(GT_T_GLOBE_SUR,l_string(std_cm,"rad"),_p->x - M_PI,_p,_reso,0,size);
 	for ( i = 0 ; i < size ; i ++ , _p ++ , _reso ++ ) {
 		if ( *_reso < 0 )
 			continue;
@@ -1186,7 +1186,9 @@ GROUP_SET_LIST * gsl;
 
 
 void
-adjust_lambda(L_CHAR * unit,REAL1 min_point,
+adjust_lambda(
+	int geo_type,
+	L_CHAR * unit,REAL1 min_point,
 	GB_POINT * pt_list,REAL1 * reso,GROUP_ID * group,int size)
 {
 REAL1 loop,max_point,x,y;
@@ -1194,6 +1196,8 @@ REAL1 vertical1,vertical2,vertical3;
 int i;
 GROUP_SET * gs;
 int y_cnt,x_cnt;
+	if ( (geo_type & GT_T_MASK) != GT_T_GLOBE_SUR )
+		return;
 	if ( group )
 		gs = new_gs(reso,group,size);
 	else 	gs = 0;
@@ -1355,6 +1359,7 @@ GROUP_ID n;
 
 	if ( r )
 		adjust_lambda(
+			r->c.geometory_type,
 			r->h.cu.unit,
 			r->h.minrect.tl.x,
 			surp->ptr,
