@@ -92,7 +92,7 @@ _load_structure_tick(GBVIEW_FLAME * gf)
 {
 	if ( gf->ls_task_cnt == 0 ) {
 printf("TICK_CREATE\n");
-		create_task(load_structure_task,(int)gf,PRI_LS_TASK);
+		create_task((void(*)(TKEY))load_structure_task,(int)gf,PRI_LS_TASK);
 	}
 	return 0;
 }
@@ -210,7 +210,7 @@ ss_printf("ls wakeup %i\n",get_tid());
 
 			_win_unlock(gf,wf);
 			wf_unlock(gf);
-
+//ss_printf("BROWSE RESOURCE %ls\n",get_url_str2(&draw->h.entry));
 			browse_resource(draw,&bw);
 
 			wf_check_resource(gf);
