@@ -26,7 +26,9 @@
 #include	"win_flame.h"
 #include	"lock_level.h"
 
-
+#define SC_OFFSET	0
+#define SC_RATE		0.5
+#define KEY_LIMIT_RATE	0.8
 #define QUEUE_LENGTH_2 	5
 
 void gc_tick_notin_tick();
@@ -35,6 +37,7 @@ void
 gc_omx(MATRIX_TOKEN * n);
 void
 gc_omx_get(MATRIX_TOKEN * n);
+int check_use_in_wf(RESOURCE*);
 
 
 /*
@@ -71,6 +74,8 @@ typedef struct omx_token_work {
 	int			h;
 	int			e_flag;
 	int			aboat_flag;
+	INTEGER64		insert_time;
+	int			key_cnt;
 } OMX_TOKEN_WORK;
 
 typedef struct omx_dirty {
@@ -96,7 +101,7 @@ void omx_new_token(MATRIX_TOKEN *,int);
 void omx_free_token(MATRIX_TOKEN * ,int);
 void omx_close_matrix_file(MATRIX * );
 OMX_OPEN_WORK * get_omx_work(RESOURCE * r);
-void omx_trigger(int type,void *t);
+int omx_trigger(int type,void *t);
 MATRIX_TOKEN *
 omx_dirty_insert(MATRIX_TOKEN * n3,MATRIX_TOKEN * n_end);
 void _exec_dirty(int type);
@@ -124,11 +129,11 @@ void mx_task2();
 	omx_lock = new_lock(LL_OMX);
 
 	memset(&mx_que1,0,sizeof(SYS_QUEUE));
-	mx_que1.flags = QF_STACK;
+	mx_que1.flags = QF_FIFO;
 	mx_que1.gc_func = gc_omx;
 	mx_que1.gc_get = gc_omx_get;
 	mx_que1.key_func = mx_task1;
-	mx_que1.pri = PRI_FETCH_MOST_STRONG;
+	mx_que1.pri = PRI_FETCH_STRONG_RES_MATRIX;
 	setup_queue(&mx_que1);
 
 	memset(&mx_que2,0,sizeof(SYS_QUEUE));
@@ -138,7 +143,7 @@ void mx_task2();
 	mx_que2.key_func = mx_task2;
 	mx_que2.total_limit = QUEUE_LENGTH_2*2;
 	mx_que2.key_limit = QUEUE_LENGTH_2;
-	mx_que2.pri = PRI_FETCH_MOST_STRONG;
+	mx_que2.pri = PRI_FETCH_STRONG_RES_MATRIX;
 	setup_queue(&mx_que2);
 	create_task(__exec_dirty,0,PRI_FETCH_MOST_STRONG);
 }
@@ -208,9 +213,12 @@ ss_printf("OPNE OLD MATRIX\n");
 	if ( ow->r->h.cache_file_id ) {
 
 		m->filename = get_filepath_from_file_id(ow->r->h.cache_file_id,".mtx");
-		err = load_matrix_header(m,O_RDWR,0644);
-
+		err = load_matrix_header(m,O_RDWR|O_CREAT,0644);
 
+		if ( err == ME_CANNOT_OPEN_FILE ) {
+			err = load_matrix_header(m,O_RDWR|O_CREAT|O_TRUNC,0644);
+ss_printf("OLD_MATRIX RETRY ERR %i\n",err);
+		}
 		if ( err == 0 ) {
 			memset(&p,0,sizeof(p));
 			p.write_file = matrix_standard_write_file;
@@ -235,6 +243,8 @@ ss_printf("OPNE OLD MATRIX\n");
 	p.total_levels = ow->r->pr64.max_level;
 	p.modify_time = ow->r->h.modify;
 	p.flags = MPF_CACHE_FILE;
+	if ( ow->r->h.flags & RF_REQUIRED_PERMISSION )
+		p.flags |= MPF_PERMISSION;
 	p.dim = 2;
 	p.pri_area[MI_FETCH_1_TP] = PR_UP + 0x2000000;
 	p.pri_area[MI_FETCH_1_MD] = PR_UP + 0x2000000;
@@ -734,6 +744,7 @@ int tim;
 void
 _exec_dirty(int data)
 {
+//ss_printf("EXEC DIRTY\n");
 	wakeup_task((int)__exec_dirty);
 }
 
@@ -742,7 +753,6 @@ void
 insert_dirty(RESOURCE * r,GB_RECT * rct)
 {
 OMX_DIRTY * d;
-int t;
 
 //ss_printf("OLD_MATRIX INSERT DIRTY\n");
 	lock_task(omx_lock);
@@ -753,13 +763,10 @@ int t;
 	dirty_list = d;
 	wakeup_task((int)__exec_dirty);
 	unlock_task(omx_lock,"_exec_dirty");
-	t = get_xltime();
-//	if ( t - dirty_time >= 2 )
-		_exec_dirty(0);
 }
 
 
-void
+int
 omx_trigger(int type,void * data)
 {
 I_RECT ir;
@@ -773,6 +780,7 @@ OMX_OPEN_WORK * ow2;
 MATRIX * m;
 MATRIX_NODE * n;
 MATRIX_TOKEN * n2;
+extern int spiral_count;
 
 	switch ( type ) {
 	case TRT_MATRIX_STANBY:
@@ -785,11 +793,12 @@ MATRIX_TOKEN * n2;
 		rct.br.y = m->pixel_size[1] / dpm;
 		insert_dirty(r,&rct);
 		_exec_dirty(1);
-		return;
+		return 0;
 	case TRT_FINISH:
 		_exec_dirty(1);
-		return;
+		return 0;
 	case TRT_TOKEN:
+//ss_printf("TRT_TOKEN trigger\n");
 		n2 = data;
 
 	//		tn = n2->tn;
@@ -818,31 +827,38 @@ MATRIX_TOKEN * n2;
 		rct.tl.y = ir.tl.y / dpm;
 		rct.br.x = ir.br.x / dpm;
 		rct.br.y = ir.br.y / dpm;
-	/*
-		tc = n2->tc;
-		if ( n2->req_level <= tn->level ||
-				tc->rt_level - n2->req_level 
-				> 2*(tc->rt_level - tn->level) )
-	*/
-	/*
-		if ( n->req_level <= n->dim_code[0] ||
-				m->total_levels - n->req_level
-				> 2*(m->total_levels - n->dim_code[0]) ) {
-
-	*/
-	/*
-			wf_insert_dirty_rect(r,
-				&rct,WFF_LUSTER_DIRTY,120,10);
-	*/
-
+//ss_printf("TOKEN DIRTY\n");
 			insert_dirty(r,&rct);
 
 	//	}
 		break;
+	case TRT_CANCEL:
+		n2 = data;
+		m = (n =n2->process_node)->matrix;
+		ow2 = m->open_work;
+		r = ow2->r;
+		if ( (MI_FETCH_1_TP <= n2->access_target_id &&
+			n2->access_target_id <= MI_FETCH_2_BT) ||
+			(MI_VISU_1_TP <= n2->access_target_id &&
+			n2->access_target_id <= MI_VISU_2_BT) ) {
+
+			if ( r->h.loop_no - n2->process_node->process_token_key
+					> spiral_count*SC_RATE + SC_OFFSET )
+{
+/*
+ss_printf("OLD MATRIX-CANCEL %p %i %i %i %ls\n",n2, r->h.loop_no,n2->process_node->process_token_key,
+r->h.loop_no - n2->process_node->process_token_key,get_url_str2(&r->h.entry));
+*/
+				return -1;
+}
+			else	return 0;
+		}
+		else	return 0;
+
 	default:
 		er_panic("omx_trigger");
 	}
-
+	return 0;
 }
 
 void
@@ -867,6 +883,11 @@ OMX_OPEN_WORK * ow1,* ow2;
 MATRIX_NODE * mn2;
 MATRIX_SEXP * s1;
 XL_SEXP * test;
+int tim;
+float limit;
+RESOURCE * lost_r;
+INTEGER64 lost_time;
+extern int spiral_count;
 
 	que = (SYS_QUEUE *)GET_TKEY(d);
 	key = touch_qkey(que);
@@ -881,13 +902,16 @@ XL_SEXP * test;
 	nr_time = 0;
 	cr_err_count = 0;
 	cr_err_count_flag = 0;
+
+	lost_r = 0;
+	lost_time = 0;
+
 	for ( ; ; ) {
 		gc_push(0,0,"tc_task2");
 		n1 = delete_queue(que,sq_key_cond,key,0);
 		if ( n1 == 0 )
 			break;
 ss_printf("GET SESSION (%x)\n",n1);
-check_point(n1);
 /*
 		if ( cr_err_count_flag && cr_err_q > 0 ) {
 			cr_err_q --;
@@ -898,6 +922,46 @@ check_point(n1);
 		r = ow1->r;
 		otw1 = n1->work;
 
+
+		tim = (get_xltime() - otw1->insert_time);
+		limit = 0;
+		if ( tim ) {
+			limit = 2.0/tim * otw1->key_cnt;
+			if ( limit < 1 )
+				set_key_limit(&mx_que2,
+					key,get_key_limit(&mx_que2,key)*KEY_LIMIT_RATE
+						+ (1-KEY_LIMIT_RATE));
+			else	
+				set_key_limit(&mx_que2,
+					key,get_key_limit(&mx_que2,key)*KEY_LIMIT_RATE
+						+ (1-KEY_LIMIT_RATE)*limit);
+		}
+		else {
+			set_key_limit(&mx_que2,
+				key,get_key_limit(&mx_que2,key)+1);
+		}
+
+/*
+ss_printf("OLD MATRIX-2 %i %i %i %ls\n", r->h.loop_no,n1->process_node->process_token_key,
+r->h.loop_no - n1->process_node->process_token_key,get_url_str2(&r->h.entry));
+		if ( r->h.loop_no - n1->process_node->process_token_key
+				> spiral_count*SC_RATE + SC_OFFSET ||
+			(lost_time == get_xltime() && lost_r == r) || 
+			(use = check_use_in_wf(r))
+				== 0 ) {
+
+			if ( use == 0 ) {
+				lost_r = r;
+				lost_time = get_xltime();
+			}
+			otw1->next = 0;
+			n1->aboat_func = 0;
+			matrix_token_error(n1);
+
+			continue;
+		}
+*/
+
 		wir = wf_search_wfid_include_resource(r);
 		if ( wir == 0 ) {
 			if ( nr_time == 0 )
@@ -1060,7 +1124,15 @@ ss_printf("CR ERR RE INSERT\n");
 			otw2->tmp = 0;
 			n1 = otw2->next;
 			n2->aboat_func = 0;
-			if ( otw2->e_flag )
+
+			ow2 = n2->process_node->matrix->open_work;
+			r = ow2->r;
+ss_printf("OLD MATRIX-3 %i %i %i %ls\n", r->h.loop_no,n2->process_node->process_token_key,
+r->h.loop_no - n2->process_node->process_token_key,get_url_str2(&r->h.entry));
+			if ( r->h.loop_no - n2->process_node->process_token_key
+					> spiral_count*SC_RATE + SC_OFFSET )
+				matrix_token_error(n2);
+			else if ( otw2->e_flag )
 				matrix_token_error(n2);
 			else	insert_matrix_access(n2,matrix_exec_cal);
 			n2 = n1;
@@ -1101,12 +1173,15 @@ XL_SEXP * data, * dd, * tmp;
 int cnt;
 I_RECT ir;
 extern int spiral_count;
-WFID_INCLUDE_RESOURCE * wir;
 
 INTEGER64 dim[2];
 INTEGER64 lev,br_x,br_y;
 MATRIX * m;
 OMX_OPEN_WORK * ow;
+int use;
+RESOURCE * lost_r;
+INTEGER64 lost_time;
+RESOURCE * r;
 
 	que = (SYS_QUEUE *)GET_TKEY(d);
 	key = touch_qkey(que);
@@ -1118,6 +1193,9 @@ OMX_OPEN_WORK * ow;
 	setup_i(xli);
 
 	ses = open_session(SEST_OPTIMIZE);
+	
+	lost_r = 0;
+	lost_time = 0;
 
 	for ( ; ; ) {
 
@@ -1130,21 +1208,31 @@ OMX_OPEN_WORK * ow;
 				unlock_task(matrix_lock,"tc_task");
 				break;
 			}
-check_point(n1);
 			unlock_task(matrix_lock,"tc_task");
 			ow = n1->process_node->matrix->open_work;
 			otw1 = n1->work;
-			if ( ow->r->pr64.loop_no - n1->loop_no
-					> spiral_count + 2 ||
-				(wir = wf_search_wfid_include_resource(ow->r))
+			use = 1;
+			r = ow->r;
+/*
+ss_printf("OLD MATRIX %i %i %i %ls\n", r->h.loop_no,n1->process_node->process_token_key,
+r->h.loop_no - n1->process_node->process_token_key,get_url_str2(&r->h.entry));
+*/
+			if ( r->h.loop_no - n1->process_node->process_token_key
+					> spiral_count*SC_RATE + SC_OFFSET ||
+				(lost_time == get_xltime() && lost_r == r) || 
+				(use = check_use_in_wf(r))
 					== 0 ) {
+
+				if ( use == 0 ) {
+					lost_r = ow->r;
+					lost_time = get_xltime();
+				}
 				otw1->next = 0;
 				n1->aboat_func = 0;
 				matrix_token_error(n1);
 
 				continue;
 			}
-			free_wfid_include_resource(wir);
 			cnt ++;
 			otw1->next = n2;
 			n2 = n1;
@@ -1165,14 +1253,14 @@ check_point(n1);
 			lev = _n->process_node->dim_code[0];
 			m = _n->process_node->matrix;
 			dim[0] = (dim[0] >> (lev * m->dim_divide[0])) &
-					(- (1<<m->block_size[0]));
+					(- (((INTEGER64)1)<<m->block_size[0]));
 			dim[1] = dim[1] >> (lev * m->dim_divide[1]) &
-					(- (1<<m->block_size[1]));
+					(- (((INTEGER64)1)<<m->block_size[1]));
 
-			br_x = dim[0] + (1<<m->block_size[0]);
+			br_x = dim[0] + (((INTEGER64)1)<<m->block_size[0]);
 			if ( br_x > m->pixel_size_list[lev][0] )
 				br_x = m->pixel_size_list[lev][0];
-			br_y = dim[1] + (1<<m->block_size[1]);
+			br_y = dim[1] + (((INTEGER64)1)<<m->block_size[1]);
 			if ( br_y > m->pixel_size_list[lev][1] )
 				br_y = m->pixel_size_list[lev][1];
 
@@ -1225,6 +1313,9 @@ token_check(_n);
 		}
 ss_printf("REMOTE SESSION (%x) ================= \n",n3);
 check_point(n3);
+		otw3 = n3->work;
+		otw3->insert_time = get_xltime();
+		otw3->key_cnt = get_key_count(&mx_que2,key);
 		insert_queue(&mx_que2,n3,1);
 
 		gc_pop(0,0);
@@ -1259,14 +1350,18 @@ MATRIX_NODE * n;
 int b;
 OMX_OPEN_WORK * ow;
 OMX_TOKEN_WORK * ww;
+MATRIX_SEXP * sd;
 	lock_task(matrix_lock);
-check_point(t);
 	m = t->process_node->matrix;
 	ow = m->open_work;
-	t->loop_no = ow->r->pr64.loop_no;
+//	t->loop_no = ow->r->h.loop_no;
 
-	if ( t->process_node->channel[CH_NET].data )
-		er_panic("old_matrix_read_net");
+	if ( t->process_node->channel[CH_NET].data ) {
+//		er_panic("old_matrix_read_net");
+		sd = t->process_node->channel[CH_NET].data;
+		t->process_node->channel[CH_NET].data = 0;
+		sd->del = 1;
+	}
 
 	t->h.key = get_server_key(&ow->r->h.target,0);
 	t->func = 0;
@@ -1292,7 +1387,6 @@ check_point(t);
 		er_panic("_get_matrix_access");
 	}
 //	t->h.pri = pri;
-check_point(t);
 	insert_queue(&mx_que1,t,0);
 	unlock_task(matrix_lock,"old_matrx_read_net");
 	return 1;
