@@ -1047,6 +1047,13 @@ int er_code;
 			return;
 		mtx = r->h.mtx;
 	}
+	if ( mtx->flags & MXF_ERR_DETECTED ) {
+			r->h.mtx = 0;
+		close_matrix(mtx);
+		gv_new_luster_option(r,0);
+		if ( r->h.mtx == 0 )
+			return;
+	}
 	div = ((int)1)<<mtx->dim_divide[0];
 	
 	ip_list = 0;
@@ -1532,6 +1539,14 @@ int err;
 			return;
 		mtx = r->h.mtx;
 	}
+	if ( mtx->flags & MXF_ERR_DETECTED ) {
+		r->h.mtx = 0;
+		close_matrix(mtx);
+		gv_new_luster_option(r,0);
+		if ( r->h.mtx == 0 )
+			return;
+		mtx = r->h.mtx;
+	}
 ss_printf("DUMMY MATRIX OK\n");
 	div = ((int)1)<<mtx->dim_divide[0];
 	start = d_alloc(sizeof(INTEGER64)*(r->h.mtx->p.dim+1));
