@@ -380,6 +380,13 @@ int clr;
 		if ( r->h.mtx == 0 )
 			return 0;
 	}
+	if ( m->flags & MXF_ERR_DETECTED ) {
+			r->h.mtx = 0;
+		close_matrix(m);
+		gv_new_luster_option(r,0);
+		if ( r->h.mtx == 0 )
+			return 0;
+	}
 	clr = 0;
 	if ( cache->n ) {
 		if ( cache->level == lev &&
@@ -685,16 +692,16 @@ REAL1 res;
 INTEGER64 xx,yy;
 REAL1 dpm1,dpm2;
 INTEGER64 level;
-float d1,d2,d;
+REAL1 d;
 unsigned long cc;
 INTEGER64 lev;
-int lev_cnt;
+INTEGER64 lev_cnt;
 MTX_CACHE cache;
-int tim,tim2;
+INTEGER64 tim,tim2;
 
 MATRIX * mtx;
-float divide;
-int int_divide;
+REAL1 divide;
+INTEGER64 int_divide;
 
 
 	if ( r->pr64.exit_lock )
@@ -707,6 +714,14 @@ int int_divide;
 			return;
 	}
 	mtx = r->h.mtx;
+	if ( mtx->flags & MXF_ERR_DETECTED ) {
+		r->h.mtx = 0;
+		close_matrix(mtx);
+		gv_new_luster_option(r,0);
+		if ( r->h.mtx == 0 )
+			return;
+		mtx = r->h.mtx;
+	}
 
 	divide = 1;
 	for ( i = 0 ; i < mtx->dim_divide[0] ; i ++ )
@@ -720,8 +735,6 @@ int int_divide;
 	dpm1 = r->pr64.dpm;
 	dpm2 = dpm1;
 	level = 0;
-	d1 = 1;
-	d2 = 0;
 
 	res = -1;
 	for ( i = 0 ; i < dw->size ; i ++ ) {
@@ -839,7 +852,6 @@ int int_divide;
 		unlock_node(cache.n,0);
 	}
 	
-
 	unlock_task(luster_lock,"draw_pixel");
 
 }
