@@ -523,8 +523,10 @@ int f;
 			i ++;
 			f = 1;
 		}
-	if ( f )
+	if ( f ) {
+//		_wf_set_delay_redraw(gf);
 		new_tick((void(*)(int))_wf_set_redraw,0,(int)gf);
+	}
 }
 
 
@@ -837,6 +839,7 @@ xx,yy,t,b,l,r);
 				break;
 		}
 		sc ++;
+//		_wf_set_delay_redraw_exec(gf);
 	}
 	spiral_count = sc;
 end:
