@@ -35,10 +35,11 @@
 #include	"radar.h"
 #include	"h_layering.h"
 #include	"xlerror.h"
+#include	"heuristics_win_frame.h"
 
 #define CURRENT_SWITCH_TIMERAG	2
 #define ZONBIE_TIMEOUT	120
-
+#define MOVE_TIMER_LIMIT	30
 
 void ov_default();
 
@@ -72,11 +73,9 @@ GBVIEW_PLOT_ELEMENT *  _wf_plot_element(GBVIEW_FLAME * gf,WF_ID id);
 void
 init_win_flame()
 {
-	SG_TITLE;
-	
-	sg("GBVIEW_FLAME*","gbview_flame_list",&gbview_flame_list);
 
 	init_h_layering();
+	init_hwf();
 
 	gbview_flame_lock = new_lock(LL_WF);
 /* init GBVIEW_FLAME
@@ -142,6 +141,43 @@ retry:
 	return system_gfid;
 }
 
+void
+move_timer_tick(int _gf)
+{
+INTEGER64 tim;
+GBVIEW_FLAME * gf;
+RADAR_CACHE * rc;
+	gf = (GBVIEW_FLAME*)_gf;
+	tim = get_xltime();
+	if ( gf->move_timer == 0 )
+		return;
+	if ( (tim - gf->move_timer) < MOVE_TIMER_LIMIT ) {
+		change_tick(MOVE_TIMER_LIMIT- (tim - gf->move_timer));
+		return;
+	}
+
+ss_printf("move_time_tick=====\n");
+	rc = (RADAR_CACHE*)gf->radar_ptr;
+	rc->last_loading_time = 0;
+	wakeup_loading_task(rc);
+	gf->move_timer = 0;
+	change_tick(-1);
+}
+
+void
+_set_move_timer(GBVIEW_FLAME* gf)
+{
+/* INVALID
+	if ( gf->move_timer == 0 ) {
+		gf->move_timer = get_xltime();
+		new_tick(move_timer_tick,MOVE_TIMER_LIMIT,(int)gf);
+	}
+	else {
+		gf->move_timer = get_xltime();
+	}
+*/
+}
+
 GBVIEW_FLAME *
 _wf_new_gf()
 {
@@ -164,6 +200,7 @@ GBVIEW_FLAME * ret;
 	ret->id = _get_new_gfid();
 
 	ret->status = GVFS_IDLE;
+	ret->move_timer = 0;
 
 	ret->next = gbview_flame_list;
 	gbview_flame_list = ret;
@@ -325,6 +362,8 @@ void
 wf_apply_all_gf(int (*func)(),void * work,int status,int lock_flag)
 {
 GBVIEW_FLAME * gf;
+int pri;
+	pri = push_pri(PRI_DRAW);
 	lock_task(gbview_flame_lock);
 	for ( gf = gbview_flame_list ; gf ; gf = gf->next ) {
 		if ( gf->status & status ) {
@@ -340,6 +379,7 @@ GBVIEW_FLAME * gf;
 		}
 	}
 	unlock_task(gbview_flame_lock,"wf_apply_all_gf");
+	change_pri(0,pri);
 }
 
 typedef struct f2outlist {
@@ -797,7 +837,7 @@ int ret;
 		return 1;
 /*
 	wf_lock(gf);
-	ZONBIE_CHECK(gf,wf_unlock,-1);
+	ZONBIE_CHECK(gf,wf_unlock(gf),-1);
 */
 	ret = _wf_active_check(wf);
 //	wf_unlock(gf);
@@ -811,7 +851,7 @@ WIN_FLAME * wf;
 int ret;
 
 	wf_lock(gf);
-	ZONBIE_CHECK(gf,wf_unlock,-1);
+	ZONBIE_CHECK(gf,wf_unlock(gf),-1);
 
 	wf = _get_wf_ptr(gf,id);
 	if ( wf == 0 )
@@ -829,7 +869,7 @@ int ret;
 WIN_FLAME * wf, * wf_id;
 int cnt;
 	wf_lock(gf);
-	ZONBIE_CHECK(gf,wf_unlock,1)
+	ZONBIE_CHECK(gf,wf_unlock(gf),1)
 
 	if ( id == 0 ) {
 		if ( gf->flame == 0 || gf->flame->next == 0 )
@@ -872,7 +912,7 @@ wf_get(GBVIEW_FLAME * gf,int no)
 WF_ID ret;
 WIN_FLAME * wf;
 	wf_lock(gf);
-	ZONBIE_CHECK(gf,wf_unlock,0);
+	ZONBIE_CHECK(gf,wf_unlock(gf),0);
 
 
 	for ( wf = gf->flame ; wf && no > 0 ; wf = wf->next , no -- );
@@ -930,7 +970,7 @@ search_resource_list(GBVIEW_FLAME * gf,RESOURCE *r,L_CHAR * target,int create_fl
 {
 GV_RESOURCE_LIST * ret;
 	wf_lock(gf);
-	ZONBIE_CHECK(gf,wf_unlock,0);
+	ZONBIE_CHECK(gf,wf_unlock(gf),0);
 	ret = _search_resource_list(gf,r,target,create_flag);
 	wf_unlock(gf);
 	return ret;
@@ -1201,6 +1241,7 @@ I_RECT rr;
 	for ( xx = xmin ; xx < xmax ; xx ++ )
 		for ( yy = ymin ; yy < ymax ; yy ++ )
 			gf->redraw[xx + gf->win_width*yy] = RP_REDRAW;
+//	_wf_set_delay_redraw(gf);
 	new_tick((void(*)(int))_wf_set_redraw,0,(int)gf);
 }
 
@@ -1252,7 +1293,6 @@ int df;
 		wf_unlock(gf);
 		return 0;
 	}
-
 	for ( si = wf->sym_i ; si ; si = si->next ) {
 		if ( si->rno != rno || si->code != code ||
 			si->sub != sub )
@@ -1573,7 +1613,7 @@ wf_plot_element(GBVIEW_FLAME * gf,WF_ID id)
 {
 GBVIEW_PLOT_ELEMENT * ret;
 	wf_lock(gf);
-	ZONBIE_CHECK(gf,wf_unlock,0)
+	ZONBIE_CHECK(gf,wf_unlock(gf),0)
 	ret = _wf_plot_element(gf,id);
 	wf_unlock(gf);
 	return ret;
@@ -1585,7 +1625,7 @@ wf_all_plot_element(GBVIEW_FLAME * gf)
 GBVIEW_PLOT_ELEMENT * ret, * _r,*_r2;
 WIN_FLAME * wf;
 	wf_lock(gf);
-	ZONBIE_CHECK(gf,wf_unlock,0)
+	ZONBIE_CHECK(gf,wf_unlock(gf),0)
 	ret = 0;
 	for ( wf = gf->flame ; wf ; wf = wf->next ) {
 		if ( wf->flags & WFF_FREE )
@@ -1611,6 +1651,8 @@ void
 wf_wait_stable(GBVIEW_FLAME * gf)
 {
 WIN_FLAME * wf;
+int pri;
+	pri = push_pri(PRI_DRAW);
 	for ( ; ; ) {
 		wf_lock(gf);
 		ZONBIE_CHECK_BREAK(gf,wf_unlock(gf));
@@ -1624,6 +1666,7 @@ WIN_FLAME * wf;
 		new_tick(_wf_wait_stable,-1,(int)gf);
 		sleep_task((int)gf,SEM_NULL);
 	}
+	change_pri(0,pri);
 }
 
 void
@@ -1641,6 +1684,7 @@ ss_printf("FREE %s(%i) %ls\n",__file,__line,get_url_str2(&wf->draw->h.entry));
 //	_wakeup_at_exit_task_wp(gf);
 	_wakeup_at_redraw_wp(gf);
 	_call_flame_insert_event(gf);
+	wakeup_hwf(gf,HWFF_TYPE_TO_FRAME);
 }
 
 void
@@ -1673,7 +1717,8 @@ void
 xx_free_win_flame(GBVIEW_FLAME * gf,WF_ID wfid,char * __file,int __line)
 {
 WIN_FLAME * wf;
-
+int pri;
+	pri = push_pri(PRI_DRAW);
 	wf_lock(gf);
 
 	wf = _get_wf_ptr(gf,wfid);
@@ -1683,6 +1728,7 @@ WIN_FLAME * wf;
 	}
 	_xx_free_win_flame(gf,wf,__file,__line);
 	wf_unlock(gf);
+	change_pri(0,pri);
 }
 
 
@@ -1732,6 +1778,7 @@ I_RECT rr;
 	insert_redraw_r(gf,&rr);
 	for ( i = 0 ; i < gf->win_height*gf->win_width ; i ++ )
 		gf->redraw[i] = RP_REDRAW;
+//	_wf_set_delay_redraw(gf);
 	new_tick((void(*)(int))_wf_set_redraw,0,(int)gf);
 }
 
@@ -1750,6 +1797,61 @@ void *	d_arg;
 		(*d_event)(gf,ET_REDRAW,0,d_arg);
 }
 
+void
+_wf_set_delay_redraw_flag(GBVIEW_FLAME * gf,int fg)
+{
+	gf->redraw_delay_flag = fg;
+	_wf_set_delay_redraw_exec(gf);
+}
+
+void
+wf_set_delay_redraw_flag(GBVIEW_FLAME * gf,int fg)
+{
+	wf_lock(gf);
+	_wf_set_delay_redraw_flag(gf,fg);
+	wf_unlock(gf);
+}
+
+void
+_wf_set_delay_redraw(GBVIEW_FLAME * gf)
+{
+	if ( gf->redraw_delay_flag ) {
+		gf->redraw_flag = 1;
+	}
+	else {
+		new_tick((void(*)(int))_wf_set_redraw,0,(int)gf);
+	}
+}
+
+void
+wf_set_delay_redraw(GBVIEW_FLAME * gf)
+{
+	wf_lock(gf);
+	_wf_set_delay_redraw(gf);
+	wf_unlock(gf);
+}
+
+void
+_wf_set_delay_redraw_exec(GBVIEW_FLAME * gf)
+{
+	if ( gf->redraw_flag ) {
+		gf->redraw_flag = 0;
+ss_printf("_wf_set_delay_redraw_exec\n");
+		new_tick((void(*)(int))_wf_set_redraw,0,(int)gf);
+	}
+}
+
+
+void
+wf_set_delay_redraw_exec(GBVIEW_FLAME * gf)
+{
+	wf_lock(gf);
+	_wf_set_delay_redraw_exec(gf);
+	wf_unlock(gf);
+}
+
+
+
 WIN_FLAME *
 _wf_get_current(GBVIEW_FLAME * gf)
 {
@@ -1761,14 +1863,17 @@ wf_get_current(GBVIEW_FLAME * gf)
 {
 WF_ID ret;
 WIN_FLAME * wf;
+int pri;
+	pri = push_pri(PRI_DRAW);
 	wf_lock(gf);
-	ZONBIE_CHECK(gf,wf_unlock,0)
+	ZONBIE_CHECK(gf,(wf_unlock(gf),change_pri(0,pri)),0)
 	wf = _wf_get_current(gf);
 
 	if ( wf )
 		ret = wf->id;
 	else	ret = 0;
 	wf_unlock(gf);
+	change_pri(0,pri);
 	return ret;
 }
 
@@ -2070,7 +2175,7 @@ wf_set_current_now(GBVIEW_FLAME * gf,WF_ID n)
 int ret;
 WIN_FLAME * now;
 	wf_lock(gf);
-	ZONBIE_CHECK(gf,wf_unlock,0);
+	ZONBIE_CHECK(gf,wf_unlock(gf),0);
 	now = _get_wf_ptr(gf,n);
 	ret = _wf_set_current(gf,now,0);
 	wf_unlock(gf);
@@ -2084,7 +2189,7 @@ wf_set_current_target(GBVIEW_FLAME * gf,WF_ID id)
 int ret;
 WIN_FLAME * wf;
 	wf_lock(gf);
-	ZONBIE_CHECK(gf,wf_unlock,0);
+	ZONBIE_CHECK(gf,wf_unlock(gf),0);
 	wf = _get_wf_ptr(gf,id);
 	if ( wf )
 		ret = _wf_set_current(gf,0,wf);
@@ -2312,6 +2417,7 @@ int dx,dy;
 	wf_move_redraw_y(gf,dy);
 	assert_ls(gf,LSF_MOVE);
 	gf->move_flag = 1;
+	_set_move_timer(gf);
 	_wakeup_win_manage(gf->radar_ptr);
 
 	return _get_env_seq(gf);
@@ -2395,6 +2501,7 @@ I_RECT ir;
 	gf->redraw_plane = _rpp;
 	assert_ls(gf,LSF_MOVE);
 	gf->move_flag = 1;
+	_set_move_timer(gf);
 	_wakeup_win_manage(gf->radar_ptr);
 
 	return _get_env_seq(gf);
@@ -2407,7 +2514,7 @@ wf_move(GBVIEW_FLAME * gf,VPOINT from,VPOINT to)
 int ret;
 
 	wf_lock(gf);
-	ZONBIE_CHECK(gf,wf_unlock,0)
+	ZONBIE_CHECK(gf,wf_unlock(gf),0)
 
 	switch ( gf->flame_base_display_map.type ) {
 	case 0:
@@ -2424,8 +2531,10 @@ int ret;
 	}
 	if ( gf->history_enable )
 		gf->history_flag = 1;
+//	_wf_set_delay_redraw(gf);
 	wf_unlock(gf);
 	new_tick((void(*)(int))_wf_set_redraw,0,(int)gf);
+	wakeup_hwf(gf,HWFF_TYPE_TO_FRAME);
 	return ret;
 }
 
@@ -2502,7 +2611,7 @@ int ret;
 VRECT rr;
 
 	wf_lock(gf);
-	ZONBIE_CHECK(gf,wf_unlock,0);
+	ZONBIE_CHECK(gf,wf_unlock(gf),0);
 
 	gf->flame_base_resolution *= zoom;
 
@@ -2534,8 +2643,10 @@ VRECT rr;
 	gf->move_flag = 1;
 	if ( gf->history_enable )
 		gf->history_flag = 1;
+//	_wf_set_delay_redraw(gf);
 	wf_unlock(gf);
 	assert_ls(gf,LSF_ZOOM);
+	_set_move_timer(gf);
 	_wakeup_win_manage(gf->radar_ptr);
 
 	if ( p ) {
@@ -2548,6 +2659,7 @@ VRECT rr;
 		d_f_ree(index_y);
 	}
 	new_tick((void(*)(int))_wf_set_redraw,0,(int)gf);
+	wakeup_hwf(gf,HWFF_TYPE_TO_FRAME);
 	return ret;
 }
 
@@ -2651,6 +2763,7 @@ WIN_FLAME * wf;
 	_calc_pitch(gf,1);
 	_change_redraw(gf,old_width,old_height);
 
+//	_wf_set_delay_redraw(gf);
 	new_tick((void(*)(int))_wf_set_redraw,0,(int)gf);
 
 	_ex_unlock(gf);
@@ -2891,7 +3004,7 @@ wf_redraw(GBVIEW_FLAME * gf,GBVIEW_PLANE * p,int x,int y,int w,int h)
 int ret;
 
 	wf_lock(gf);
-	ZONBIE_CHECK(gf,wf_unlock,0)
+	ZONBIE_CHECK(gf,wf_unlock(gf),0)
 
 	_ex_lock(gf);
 
@@ -2911,7 +3024,7 @@ WIN_FLAME * wf, * ret_wf;
 /* retry: */
 int flags;
 	wf_lock(gf);
-	ZONBIE_CHECK(gf,wf_unlock,0)
+	ZONBIE_CHECK(gf,wf_unlock(gf),0)
 
 retry2:
 	_reset_task_nos(gf);
@@ -2930,7 +3043,7 @@ retry2:
 				ret_wf = wf;
 		}
 	}
-	ZONBIE_CHECK(gf,wf_unlock,0)
+	ZONBIE_CHECK(gf,wf_unlock(gf),0)
 	if ( ret_wf ) {
 		flags = ret_wf->flags;
 		ret_wf->flags &= ~WFF_LS_DIRTY;
@@ -3220,7 +3333,7 @@ int ses;
 	if ( u.proto == 0 || u.db == 0 || u.server == 0 || u.resource == 0 )
 		return 0;
 	wf_lock(gf);
-	ZONBIE_CHECK(gf,wf_unlock,0);
+	ZONBIE_CHECK(gf,wf_unlock(gf),0);
 
 	for ( wf = gf->flame ; wf ; wf = wf->next ) {
 		if ( wf->flags & WFF_FREE )
@@ -3338,6 +3451,7 @@ int ses;
 end1:
 	assert_ls(gf,LSF_ZOOM);
 	gf->move_flag = 1;
+	_set_move_timer(gf);
 	_wakeup_win_manage(gf->radar_ptr);
 	wf_unlock(gf);
 end:
@@ -3478,7 +3592,7 @@ wf_purge_select(GBVIEW_FLAME * gf)
 {
 int ret;
 	wf_lock(gf);
-	ZONBIE_CHECK(gf,wf_unlock,0);
+	ZONBIE_CHECK(gf,wf_unlock(gf),0);
 
 	if ( _wf_purge_select(gf) < 0 )
 		ret = 0;
@@ -3794,8 +3908,10 @@ wf_get_resource(GBVIEW_FLAME * gf,WF_ID id)
 {
 RESOURCE * ret;
 WIN_FLAME * wf;
+int pri;
+	pri = push_pri(PRI_DRAW);
 	wf_lock(gf);
-	ZONBIE_CHECK(gf,wf_unlock,0);
+	ZONBIE_CHECK(gf,(wf_unlock(gf),change_pri(0,pri)),0);
 
 
 	wf = _get_wf_ptr(gf,id);
@@ -3805,6 +3921,7 @@ WIN_FLAME * wf;
 	}
 	ret = wf->draw;
 	wf_unlock(gf);
+	change_pri(0,pri);
 	return ret;
 }
 
@@ -3893,7 +4010,7 @@ WIN_FLAME * wf;
 	if ( from.x == to.x && from.y == to.y )
 		return 0;
 	wf_lock(gf);
-	ZONBIE_CHECK(gf,wf_unlock,0)
+	ZONBIE_CHECK(gf,wf_unlock(gf),0)
 	
 	index = d_alloc(sizeof(VPOINT)*gf->win_width*gf->win_height);
 	wf_make_rotate_index(index,from,to,gf->win_width,gf->win_height);
@@ -3916,6 +4033,7 @@ WIN_FLAME * wf;
 		gf->history_flag = 1;
 
 	ret = _get_env_seq(gf);
+//	_wf_set_delay_redraw(gf);
 	wf_unlock(gf);
 	new_tick((void(*)(int))_wf_set_redraw,0,(int)gf);
 	return ret;
@@ -4110,7 +4228,7 @@ int ret;
 	if ( o == 0 || o->h.name == 0 )
 		return 0;
 	wf_lock(gf);
-	ZONBIE_CHECK(gf,wf_unlock,0)
+	ZONBIE_CHECK(gf,wf_unlock(gf),0)
 	rlist_len = 0;
 	for ( wf = gf->flame ; wf ; wf = wf->next ) {
 		if ( wf->flags & (WFF_FREE|WFF_HIDE) )
@@ -4150,12 +4268,13 @@ int n;
 RESOURCE * r;
 int er;
 GBVIEW_LAYER_STATUS * ls, ** lsp;
+int pri;
 
-
+	pri = push_pri(PRI_DRAW);
 	wf_lock(gf);
 
 	wf_init_status(e);
-	ZONBIE_CHECK_VOID(gf,wf_unlock(gf))
+	ZONBIE_CHECK_VOID(gf,(wf_unlock(gf),change_pri(0,pri)))
 
 	e->width = gf->win_width;
 	e->height = gf->win_height;
@@ -4259,6 +4378,7 @@ GBVIEW_LAYER_STATUS * ls, ** lsp;
 	lock_task(in_lock);
 	e->indicate = _copy_indicate(gf);
 	unlock_task(in_lock,"wf_status");
+	change_pri(0,pri);
 }
 
 void
@@ -4322,6 +4442,7 @@ WIN_FLAME * wf;
 			_ch_unlock(&gf->_wf_lock);
 			gf->status = GVFS_ZONBIE_2;
 			gf->zonbie_timeout = get_xltime() + ZONBIE_TIMEOUT;
+			gf->move_timer = 0;
 			break;
 		case GVFS_ZONBIE_2:
 			if ( gf->zonbie_timeout < get_xltime() )
@@ -4391,7 +4512,7 @@ int change,st_change;
 int shift;
 int old_width,old_height;
 	wf_lock(gf);
-	ZONBIE_CHECK(gf,wf_unlock,0)
+	ZONBIE_CHECK(gf,wf_unlock(gf),0)
 	change = 0;
 	st_change = 0;
 	if ( e->flags & SF_STATUS ) {
@@ -4577,7 +4698,7 @@ wf_insert_time(GBVIEW_FLAME * gf,int wfid)
 WIN_FLAME * wf;
 unsigned int ret;
 	wf_lock(gf);
-	ZONBIE_CHECK(gf,wf_unlock,0)
+	ZONBIE_CHECK(gf,wf_unlock(gf),0)
 	wf = _get_wf_ptr(gf,wfid);
 	if ( wf == 0 )
 		ret = 0;
@@ -4626,10 +4747,11 @@ int ret;
 int f;
 GB_RECT rr;
 GV_RESOURCE_LIST * ri;
-	
+int hwf_active = 0;
+
 	ret = 0;
 	wf_lock(gf);
-	ZONBIE_CHECK(gf,wf_unlock,-1);
+	ZONBIE_CHECK(gf,wf_unlock(gf),-1);
 	if ( url == 0 ) {
 		if ( mask & WFF_BASE_LOCK ) {
 			f = 0;
@@ -4664,15 +4786,22 @@ GV_RESOURCE_LIST * ri;
 			ri->flags &= ~WFF_BASE_LOCK;
 	}
 	ri = _search_resource_list(gf,0,url,1);
+	ret = -1;
+	if ( (ri->flags & WFF_USER) && (flags & WFF_USER_NEGATE) )
+		goto end;
 	ri->flags &= ~(WFF_COPY_FLAGS&mask);
 	ri->flags |= flags & WFF_COPY_FLAGS;
-	ret = -1;
+ss_printf("RI COPY FLAGS %ls %x %x - %x\n",url,ri->flags,flags,mask);
 	goto end;
 ok:
-	flags &= mask;
+//	flags &= mask;
 	f = wf->flags;
+
+	if ( (wf->flags & WFF_USER) && (flags & WFF_USER_NEGATE) )
+		goto end;
 	wf->flags &= ~(WFF_COPY_FLAGS&mask);
 	wf->flags |= flags & WFF_COPY_FLAGS;
+ss_printf("REAL COPY FLAGS %ls %x -> %x %x - %x\n",url,f,wf->flags,flags,mask);
 
 	if ( (f & WFF_HIDE) != (wf->flags & WFF_HIDE) ) {
 		rr.tl.x = rr.tl.y = 0;
@@ -4681,7 +4810,7 @@ ok:
 		
 		if ( (flags & WFF_HIDE_TIME_DISABLE) == 0 )
 			wf->hide_time = get_xltime();
-		if ( flags ) {
+		if ( flags & WFF_HIDE ) {
 			wf->flags |= WFF_HIDE;
 			set_redraw_rect(gf,&rr);
 		}
@@ -4691,7 +4820,9 @@ ok:
 			_win_flame_dirty(gf,wf,&rr,WFF_DIRTY,0);
 			force_change_task();
 		}
+ss_printf("CALL_FRAME_CHANGE\n");
 		_call_flame_change_base_event(gf);
+		hwf_active = 1;
 	}
 	if ( wf->flags & (WFF_BASE_LOCK) ) {
 		for ( wf2 = gf->flame ; wf2 ; wf2 = wf2->next ) {
@@ -4709,6 +4840,8 @@ end:
 	free_url(&u);
 end2:
 	wf_unlock(gf);
+	if ( hwf_active && (flags & WFF_USER) && (mask & WFF_USER) ) 
+		wakeup_hwf(gf,HWFF_TYPE_TO_HWF);
 	return ret;
 }
 
@@ -4717,7 +4850,7 @@ int wf_get_flags(GBVIEW_FLAME * gf,WF_ID wfid)
 int ret;
 WIN_FLAME * wf;
 	wf_lock(gf);
-	ZONBIE_CHECK(gf,wf_unlock,-1);
+	ZONBIE_CHECK(gf,wf_unlock(gf),-1);
 	wf = _get_wf_ptr(gf,wfid);
 	if ( wf == 0 )
 		ret = -1;
