@@ -30,7 +30,7 @@ extern int test_debug;
 
 #define GVHA_LOADING_STATUS
 
-#define SCAN_LEVEL_OFS		2
+#define SCAN_LEVEL_OFS		((INTEGER64)2)
 
 
 #define IGN_SEARCH_LOADING	0x00000001
@@ -305,15 +305,15 @@ int x_ofs,y_ofs;
 	n = sc->n;
 	m = sc->m;
 	x_ofs = (sc->dim_code[1]>>(sw->start[0]*m->dim_divide[0])) 
-			& ((1<<(SCAN_LEVEL_OFS*m->dim_divide[0]+m->block_size[0]))-1);
+			& ((((INTEGER64)1)<<(SCAN_LEVEL_OFS*m->dim_divide[0]+m->block_size[0]))-1);
 	y_ofs = (sc->dim_code[2]>>(sw->start[0]*m->dim_divide[1])) 
-			& ((1<<(SCAN_LEVEL_OFS*m->dim_divide[1]+m->block_size[1]))-1);
-	width = 1<<m->block_size[0];
-	height = 1<<m->block_size[1];
+			& ((((INTEGER64)1)<<(SCAN_LEVEL_OFS*m->dim_divide[1]+m->block_size[1]))-1);
+	width = ((int)1)<<m->block_size[0];
+	height = ((int)1)<<m->block_size[1];
 	rr.tl.x = 0;
 	rr.tl.y = 0;
-	rr.br.x = (_width = 2*width + (1<<(m->block_size[0]+SCAN_LEVEL_OFS*m->dim_divide[0])));
-	rr.br.y = (_height = 2*height + (1<<(m->block_size[1]+SCAN_LEVEL_OFS*m->dim_divide[1])));
+	rr.br.x = (_width = 2*width + (((INTEGER64)1)<<(m->block_size[0]+SCAN_LEVEL_OFS*m->dim_divide[0])));
+	rr.br.y = (_height = 2*height + (((INTEGER64)1)<<(m->block_size[1]+SCAN_LEVEL_OFS*m->dim_divide[1])));
 
 	switch ( sc->dir ) {
 	case MST_FIRST:
@@ -560,8 +560,8 @@ ss_printf("call-force %s %s\n",pt_dc(mx_e->c.m,_start,PTDC_NODE_ID),pt_dc(mx_e->
 	sts.event = call_spider_event;
 	sts.event_user_arg = (void*)&sw;
 	sts.default_ws = get_ws(l_string(std_cm,"DefaultStyle"));
-	sts.width = (1<<mx_e->c.m->block_size[0])*2+(1<<(mx_e->c.m->block_size[0]+SCAN_LEVEL_OFS*mx_e->c.m->dim_divide[0]));
-	sts.height = (1<<mx_e->c.m->block_size[1])*2+(1<<(mx_e->c.m->block_size[1]+SCAN_LEVEL_OFS*mx_e->c.m->dim_divide[1]));
+	sts.width = (((int)1)<<mx_e->c.m->block_size[0])*2+(((int)1)<<(mx_e->c.m->block_size[0]+SCAN_LEVEL_OFS*mx_e->c.m->dim_divide[0]));
+	sts.height = (((int)1)<<mx_e->c.m->block_size[1])*2+(((int)1)<<(mx_e->c.m->block_size[1]+SCAN_LEVEL_OFS*mx_e->c.m->dim_divide[1]));
 	wf_set_status(gf,&sts);
 	
 	sw.start = _start;
@@ -573,8 +573,8 @@ ss_printf("call-force %s %s\n",pt_dc(mx_e->c.m,_start,PTDC_NODE_ID),pt_dc(mx_e->
 	set_base_permanent_lock(wp);
 	rc_warp((RADAR_CACHE*)gf->radar_ptr,WT_WARP_POINT,0,0,0,0,0,wp,1);
 	
-	width = 1<<mx_e->c.m->block_size[0];
-	height = 1<<mx_e->c.m->block_size[1];
+	width = ((int)1)<<mx_e->c.m->block_size[0];
+	height = ((int)1)<<mx_e->c.m->block_size[1];
 	rr.tl.x = 0;
 	rr.tl.y = 0;
 	rr.br.x = 3*width;
