@@ -1231,12 +1231,12 @@ int sz;
 					return -22;
 				dir++;
 				str = get_str(&dir,&ch,dir,"_");
-				sscanf(str,"%f",&q->pos.center.x);
+				sscanf(str,REAL1_F,&q->pos.center.x);
 				d_f_ree(str);
 				if ( *dir == 0 )
 					return -5;
 				str = get_str(&dir,&ch,dir,"_");
-				sscanf(str,"%f",&q->pos.center.y);
+				sscanf(str,REAL1_F,&q->pos.center.y);
 				d_f_ree(str);
 				if ( *dir == 0 )
 					return -6;
