@@ -115,7 +115,7 @@ typedef struct pn_file_header {
 #define FT_ADDRESS_DB  21 /* address database */
 
 	char		version[8];
-#define PDB_VERSION		"0.4"
+#define PDB_VERSION		"0.6"
 
 #define ENCODING_SIZE	8
 	char		encoding[ENCODING_SIZE];
@@ -285,7 +285,7 @@ typedef struct pn_dtree_data {
 
 typedef struct floating_index {
 	unsigned int		fofs;
-	float			d;
+	REAL1			d;
 } FLOATING_INDEX;
 
 typedef struct gb_time_index {
@@ -357,7 +357,7 @@ typedef struct pn_assoc_data {
 typedef struct pn_pmd_header {
 	PN_HEADER	h;
 	char		version[8];
-#define PMD_VERSION	"0.2"
+#define PMD_VERSION	"0.3"
 
 #define PMD_UNIT_LEN	200
 	L_CHAR		unit_path[PMD_UNIT_LEN];
@@ -378,7 +378,7 @@ typedef struct pn_pmd_header {
 typedef struct pn_pmd_node {
 	PN_HEADER	h;
 	GB_RECT		minrect;
-	float		resolution;
+	REAL1		resolution;
 	GB_TIME		content_period_from;
 	GB_TIME		content_period_to;
 	unsigned int	timeout;
@@ -550,8 +550,8 @@ typedef struct pn_rs_res_header {
 /* NOTE recors
 	NOTE *		notes;
 */
-	float		visible_resolution;
-	float		limit_resolution;
+	REAL1		visible_resolution;
+	REAL1		limit_resolution;
 /* string after this structure SYSTEM and UNIT
 	COORDINATE_UNIT cu;
 */
@@ -600,7 +600,7 @@ typedef struct pn_rs_note {
 
 typedef struct pn_rs_pixel_map_r64 {
 	PN_RS_RES_HEADER	h;
-	float		dpm;
+	REAL1		dpm;
 	unsigned char	type;
 	/*
 		'P'	R64 RGB
@@ -634,7 +634,7 @@ typedef struct pn_rs_coordinate {
 /* cut
 	OVERLAY *	ov;
 */
-	float		weight;
+	REAL1		weight;
 /* string after this structure
 	L_CHAR *	mode;
 	XL_SEXP *	initial_cmd;
@@ -713,7 +713,7 @@ typedef struct pn_rs_draw_gb {
 	int		pdb_lock_msg;
 	int		pdb_lock_gc_flag;
 */
-	float		resolution;
+	REAL1		resolution;
 /* cut
 	AVT_NODE *	obj_code_tree;
 	RECT_TREE	obj_rect_tree;
@@ -731,7 +731,7 @@ typedef struct pn_rs_draw_gb {
 /* INFO_SCHEME record
 	INFO_SCHEME *	ischeme;
 */
-	float		onmap_indicate_size;
+	REAL1		onmap_indicate_size;
 	short		hit_level;
 	short		hit_outofcircle;
 
@@ -762,7 +762,7 @@ typedef struct pn_rs_triangle_set {
 	int			cnt;
 	AFFEN2D			a;
 	GB_RECT			minrect;
-	float			resolution_rate;
+	REAL1			resolution_rate;
 /* TRIANGLE records : 
 		the list of pair of PN_RS_TRIANGLE
 		1st is param_forward
@@ -773,7 +773,7 @@ typedef struct pn_rs_triangle_set {
 
 typedef struct pn_rs_map_parameter {
 	int	type;
-	float	resolution_rate;
+	REAL1	resolution_rate;
 	union {
 		GB_POINT	ptr;
 		AFFEN2D		affen;
@@ -814,7 +814,7 @@ typedef struct pn_rs_triangle {
 */
 	GB_POINT		ptp[3];
 	AFFEN2D			a;
-	float			resolution_rate;
+	REAL1			resolution_rate;
 	GB_RECT			minrect;
 } PN_RS_TRIANGLE;
 
