@@ -1069,7 +1069,7 @@ void dispose_lrw(LOAD_RESOURCE_WORK * w,int terminate);
 void test_map_reso(char* str,REAL1 * reso,int size,int data);
 
 void
-adjust_lambda(L_CHAR * unit,REAL1 min_point,
+adjust_lambda(int geo_type,L_CHAR * unit,REAL1 min_point,
 	GB_POINT * pt_list,REAL1 * reso,GROUP_ID * group,int size);
 
 /*
