@@ -139,6 +139,10 @@ typedef struct remote_session_opt {
 #define RFT_BODY	2
 } REMOTE_SESSION_OPT;
 
+#define NPS_DIRECT	1
+#define NPS_PROXY	2
+extern int native_proxy_session;
+
 typedef struct xl_result {
 	DELAY_FUNC		h;
 	struct xl_result *	next;
@@ -520,7 +524,7 @@ get_remote_version(
 	URL * u,
 	char * layer);
 L_CHAR * compose_path(L_CHAR *,L_CHAR *);
-XL_SEXP * err_handler(XL_SEXP *,XL_FILE *,int);
+XL_SEXP * err_handler(XL_SEXP *,XL_FILE *,int,L_CHAR * _stderr);
 
 /* Relational Operation flags */
 #define RO_SORT1	0x00000001
@@ -914,6 +918,20 @@ XL_SEXP * remote_session_http(
 	int ln,
 	int session_lock_enable,
 	REMOTE_SESSION_OPT *);
+XL_SEXP *
+native_proxy_send_session(
+	int * mode,
+	XLISP_ENV * env,
+	int id,
+	URL * u,
+	L_CHAR * a_agent,
+	L_CHAR * a_login_mode,
+	L_CHAR * a_center_cmd,
+	XL_SEXP * cmd,
+	XL_FILE * f,
+	int ln,
+	int session_lock_enable,
+	REMOTE_SESSION_OPT * opt);
 void init_session_xlp();
 void init_server_function(XLISP_ENV * env0,XLISP_ENV * env1);
 void init_sv_Exit(XLISP_ENV * env);
