@@ -546,6 +546,7 @@ g_ptr(int * errp,FAVT_ROOT * rt,FAVT_PTR * ptr)
 		ptr->ptr = favt_load(errp,0,rt,ptr->fofs,ptr);
 		((FAVT_NODE *)(ptr->ptr))->parent = ptr;
 	}
+	touch_ring(ptr->ptr,0);
 	return ptr->ptr;
 }
 
@@ -675,6 +676,7 @@ favt_insert(int * errp,
 {
 FAVT_NODE * ret;
 FAVT_NODE * g;
+	touch_ring(p,0);
 	if ( favtp->fofs == 0 ) {
 		if ( a->data == 0 )
 			er_panic("avt_insert(1)");
@@ -903,14 +905,19 @@ _favt_delete_small(int * errp,FAVT_ROOT *p,FAVT_PTR * ap)
 FAVT_NODE * ret;
 FAVT_NODE * g;
 	g = g_ptr(errp,p,ap);
+	favt_lock(&g->h);
 	if ( g->small.fofs ) {
 		ret = _favt_delete_small(errp,p,&g->small);
 		fset_level(errp,p,g);
+		favt_lock(&g->h);
 		return ret;
 	}
 	ret = g;
 	set_favt_ptr(ap,g->large);
 	favt_valance(errp,p,ap);
+	touch_ring(0,ret);
+
+	favt_unlock(&ret->h);
 	return ret;
 }
 
@@ -1205,13 +1212,9 @@ void
 check_cache(char * str)
 {
 FAVT_HEADER * h;
-int cnt;
 	if ( favt_h.next == 0 )
 		return;
-	cnt = 200;
-	for ( h = favt_h.next ; h != &favt_h ; h = h->next , cnt -- ) {
-		if ( cnt < 0 )
-			er_panic("chec_cache(2)");
+	for ( h = favt_h.next ; h != &favt_h ; h = h->next ) {
 		if ( h->type == PNT_FAVT_ROOT )
 			cc_root(str,(FAVT_ROOT*)h);
 		if ( h->type != PNT_FAVT_ROOT && h->type != PNT_FAVT_NODE )
