@@ -23,7 +23,7 @@
 #include	"favt64.h"
 #include	"utils.h"
 #include	"avt.h"
-
+#include	"fa_cache.h"
 
 extern void (*change_endian_favt64_table_to_host[])();
 
@@ -95,6 +95,7 @@ init_favt64()
 {
 	favt64_h.next = favt64_h.prev = &favt64_h;
 	fr64_tree = 0;
+	fac_init_general();
 }
 
 /*
@@ -321,7 +322,7 @@ PDB64 * p;
 	n->h.type = PNT_FAVT_NODE;
 	size = n->h.size = nn->data_len + sizeof(PN64_FAVT_NODE);
 	change_endian_favt64_node(n);
-	u_lseek64(p->fid,h->fofs,SEEK_SET);
+	fac_lseek(&p->fac,h->fofs,SEEK_SET);
 	fa_write64_header(p,n,size);
 	d_f_ree(n);
 	return 0;
@@ -339,7 +340,7 @@ PN64_FAVT_ROOT r;
 	r.h.type = PNT_FAVT_ROOT;
 	r.h.size = sizeof(r);
 	change_endian_favt64_root(&r);
-	u_lseek64(h->p->fid,h->fofs,SEEK_SET);
+	fac_lseek(&h->p->fac,h->fofs,SEEK_SET);
 	fa_write64_header(h->p,&r,sizeof(r));
 	return 0;
 }
