@@ -51,6 +51,8 @@ int check_filespace64(PDB64 * p);
 //U_INTEGER64 debug_target_fofs=(U_INTEGER64)56;
 U_INTEGER64 debug_target_fofs=(U_INTEGER64)-1;
 
+
+
 void
 check_pdb64(PDB64 * p)
 {
@@ -61,11 +63,11 @@ check_pdb64(PDB64 * p)
 }
 
 void
-check_access_write(int fd,U_INTEGER64 length)
+check_access_write(FAC_INFO * info,U_INTEGER64 length)
 {
 INTEGER64 start;
 
-	start = u_lseek64(fd,0,SEEK_CUR);
+	start = fac_lseek(info,0,SEEK_CUR);
 	if ( start < 0 )
 		er_panic("CAW");
 	if ( start <= debug_target_fofs && start+length > debug_target_fofs )
@@ -111,7 +113,7 @@ INTEGER64 ofs_buf;
 	if ( (p->flags & PF_DEBUG) == 0 )
 		return 0;
 	if ( cmd & DT_POS ) {
-		fofs = u_lseek64(p->fid,0,SEEK_CUR);
+		fofs = fac_lseek(&p->fac,0,SEEK_CUR);
 	}
 //ss_printf(">> %p >> %x:" I64_FORMAT " " I64_FORMAT "\n",p,cmd,fofs,size);
 	switch ( cmd & DT_CMD ) {
@@ -142,7 +144,7 @@ INTEGER64 ofs_buf;
 			break;
 		}
 ss_printf(">>> DP SEARCH %lli %lli %lli\n",fofs,_size,fofs2);
-		ofs_buf = u_lseek64(p->fid,0,SEEK_CUR);
+		ofs_buf = fac_lseek(&p->fac,0,SEEK_CUR);
 		for ( ; fofs2 < fofs ; ) {
 			p->flags |= PF_NEST_DEBUG;
 			h = get_file_record64(&fofs2,p,0,0);
@@ -156,7 +158,7 @@ ss_printf(">>> DP SEARCH %lli %lli %lli\n",fofs,_size,fofs2);
 			_size = h->size;
 			d_f_ree(h);
 		}
-		u_lseek64(p->fid,ofs_buf,SEEK_SET);
+		fac_lseek(&p->fac,ofs_buf,SEEK_SET);
 		if ( fofs == fofs2 ) {
 			dp = d_alloc(sizeof(*dp));
 			a = d_alloc(sizeof(*a));
@@ -246,7 +248,7 @@ U_INTEGER64 _len;
 		if ( len > 0x10000 )
 			_s = 0x10000;
 		else	_s = len;
-		er = u_write(fd,dp,_s);
+		er = fac_write(&p->fac,dp,_s);
 		if ( er <= 0 )
 			break;
 		len -= er;
@@ -270,7 +272,7 @@ int fd;
 int flags;
 INTEGER64 st_ofs;
 
-	st_ofs = u_lseek64(p->fid,0,SEEK_CUR);
+	st_ofs = fac_lseek(&p->fac,0,SEEK_CUR);
 
 	fd = p->fid;
 
@@ -332,7 +334,7 @@ int er,len;
 U_INTEGER64 offset,last_offset;
 int pflags;
 
-	offset = u_lseek64(p->fid,0,SEEK_END);
+	offset = fac_lseek(&p->fac,0,SEEK_END);
 	h = d;
 	hh = *h;
 	change_endian_header64(&hh);
@@ -352,6 +354,11 @@ int pflags;
 		p->flags &= ~PF_DEBUG;
 		dummy = d_alloc(DUMMY_SIZE);
 		for ( s = size - sizeof(*h) ; s > 0 ; ) {
+if ( s > size )
+{
+sleep_sec(2);
+er_panic("stop");
+}
 			if ( s > DUMMY_SIZE )
 				len = DUMMY_SIZE;
 			else	len = s;
@@ -359,7 +366,7 @@ int pflags;
 			s -= er;
 		}
 		p->flags |= pflags;
-		last_offset = u_lseek64(p->fid,0,SEEK_CUR);
+		last_offset = fac_lseek(&p->fac,0,SEEK_CUR);
 		d_f_ree(dummy);
 
 		debug_pdb64_list(p,DT_BOUNDARY,offset,last_offset - offset);
@@ -413,7 +420,7 @@ INTEGER64 read_size;
 	if ( (p->flags & PF_NEST_DEBUG) && (debug_pdb64_list(p,DT_DP_SEARCH,fofs,0) < 0) )
 		er_panic("get_file_record64 position");
 
-	length = u_lseek64(p->fid,0,SEEK_END);
+	length = fac_lseek(&p->fac,0,SEEK_END);
 	if ( length < 0 ) {
 ss_printf("get_file_record64-1 = " I64_FORMAT "\n",length);
 		return 0;
@@ -427,10 +434,10 @@ length,fofs);
 
 		debug_pdb64_list(p,DT_INSERT,fofs,0);
 
-		er = u_lseek64(p->fid,fofs,SEEK_SET);
+		er = fac_lseek(&p->fac,fofs,SEEK_SET);
 		if ( er < 0 )
 			return 0;
-		er = u_read(p->fid,&ph,sizeof(PN_HEADER));
+		er = fac_read(&p->fac,&ph,sizeof(PN_HEADER));
 
 // ph.h16.type NET
 
@@ -444,7 +451,7 @@ length,fofs);
 		}
 		switch ( h16_size ) {
 		case PHYS_SIZE32:
-			er = u_read(p->fid,&ph.h32.size,sizeof(int));
+			er = fac_read(&p->fac,&ph.h32.size,sizeof(int));
 			if ( er < sizeof(int) )
 				return 0;
 			change_endian_phys_header64_to_host(&ph);
@@ -453,7 +460,7 @@ length,fofs);
 			size = ph.h32.size;
 			break;
 		case PHYS_SIZE64:
-			er = u_read(p->fid,&ph.h64.size,
+			er = fac_read(&p->fac,&ph.h64.size,
 					sizeof(INTEGER64));
 			if ( er < sizeof(INTEGER64) )
 				return 0;
@@ -493,7 +500,7 @@ length,fofs);
 		if ( size > 0x10000 )
 			_s = 0x10000;
 		else	_s = size;
-		er = u_read(p->fid,ptr,_s);
+		er = fac_read(&p->fac,ptr,_s);
 		if ( er < _s )
 			return 0;
 		ptr += er;
@@ -523,7 +530,7 @@ U_INTEGER64 fofs;
 	p->pdb64_debug_root = 0;
 //p->flags |= PF_DEBUG;
 	debug_pdb64_list(p,DT_INSERT,0,0);
-	length = u_lseek64(p->fid,0,SEEK_END);
+	length = fac_lseek(&p->fac,0,SEEK_END);
 	if ( length == 0 ) {
 		h.h.size = sizeof(h);
 		h.h.type = PNT_FILE_HEADER;
@@ -590,6 +597,7 @@ U_INTEGER64 fofs;
 			return 0;
 		}
 	}
+	fac_flush(&p->fac);
 	return p;
 }
 
@@ -664,7 +672,7 @@ int er;
 PN64_FREELIST buf;
 	buf = *f;
 	change_endian_freelist64_to_net(&buf);
-	u_lseek64(p->fid,fofs,SEEK_SET);
+	fac_lseek(&p->fac,fofs,SEEK_SET);
 
 	debug_pdb64_list(p,DT_INSERT,fofs,sizeof(buf));
 
@@ -782,7 +790,7 @@ fit_ok:
 
 check_freelist64(p);
 
-//ss_printf("ALLOC " I64_FORMAT " " I64_FORMAT "\n",ret,h->size);
+//ss_printf("ALLOC-3 " I64_FORMAT " " I64_FORMAT "\n",ret,h->size);
 	debug_pdb64_list(p,DT_INSERT,ret,0);
 
 	return ret;
@@ -804,7 +812,7 @@ check_freelist64(p);
 
 	debug_pdb64_list(p,DT_INSERT,ret,0);
 
-//ss_printf("ALLOC " I64_FORMAT " " I64_FORMAT "\n",ret,h->size);
+//ss_printf("ALLOC-1 " I64_FORMAT " " I64_FORMAT "\n",ret,h->size);
 	return ret;
 nothing:
 
@@ -818,7 +826,7 @@ check_freelist64(p);
 
 	debug_pdb64_list(p,DT_INSERT,ret,0);
 
-//ss_printf("ALLOC " I64_FORMAT " " I64_FORMAT "\n",ret,h->size);
+//ss_printf("ALLOC-2 " I64_FORMAT " " I64_FORMAT "\n",ret,h->size);
 	return ret;
 }
 
@@ -845,7 +853,7 @@ INTEGER64 fofs;
 	fofs = fp1->fofs;
 	if ( delete_freelist64(0,p,fp) < 0 )
 		er_panic("truncate_cache64(1)");
-	ftruncate64(p->fid,fofs);
+	fac_truncate(&p->fac,fofs);
 }
 
 
@@ -856,16 +864,16 @@ PN64_FREELIST f;
 FREELIST64 * fp,* fp1,* prev;
 U_INTEGER64 length;
 
-
 check_freelist64(p);
 
 //ss_printf("FREE FILESPACE " I64_FORMAT "\n",fofs);
 	if ( fofs == 0 )
 		er_panic("free_filespace64");
 	load_freelist64(p,&f,fofs);
-	length = u_lseek64(p->fid,0,SEEK_END);
+//ss_printf("FREE FILESPACE " I64_FORMAT " " I64_FORMAT "\n",fofs,(INTEGER64)PHYS_SIZE(f.h.size));
+	length = fac_lseek(&p->fac,0,SEEK_END);
 	if ( fofs + PHYS_SIZE(f.h.size) >= length ) {
-		ftruncate64(p->fid,fofs);
+		fac_truncate(&p->fac,fofs);
 		truncate_check64(p,fofs);
 
 check_freelist64(p);
@@ -895,14 +903,12 @@ check_freelist64(p);
 next_nothing:
 
 
-
 	if ( fofs + PHYS_SIZE(f.h.size) == fp->fofs )
 		goto stack_on;
 	goto only_one;
 three_type:
 
 
-
 	if ( fofs + PHYS_SIZE(f.h.size) == fp->fofs ) {
 		if ( fp1->fofs + PHYS_SIZE(fp1->f.h.size) == fofs )
 			goto sandwitch;
@@ -913,8 +919,6 @@ three_type:
 	else	goto only_one;
 large:
 
-
-
 	if ( fp1->fofs + PHYS_SIZE(fp1->f.h.size) == fofs )
 		goto hang_on;
 	else if ( fp1->fofs + PHYS_SIZE(fp1->f.h.size) > fofs ) {
@@ -931,6 +935,7 @@ large:
 
 only_one:
 
+check_freelist64(p);
 
 
 	insert_freelist64(p,fp,fofs,f.h.size);
@@ -989,7 +994,7 @@ U_INTEGER64 length;
 	if ( fp == 0 )
 		return -1;
 	header = fp->fofs;
-	length = u_lseek64(p->fid,0,SEEK_END);
+	length = fac_lseek(&p->fac,0,SEEK_END);
 	prev = 0;
 	for ( ; fp ; prev = fp->fofs , fp = next_freelist64(p,fp) ) {
 		if ( fp == 0 )
@@ -1028,6 +1033,7 @@ U_INTEGER64 length;
 		if (  prev <= fp->fofs ) {
 			fprintf(stderr,"prv" I64_FORMAT " ofs" I64_FORMAT " typ%x siz" I64_FORMAT "\n",
 				prev,fp->fofs,fp->f.h.type,fp->f.h.size);
+sleep_sec(2);
 			er_panic("check_freelist64(2)");
 		}
 		if ( fp->fofs + fp->f.h.size >= prev ) {
@@ -1051,7 +1057,7 @@ int er;
 unsigned short s16;
 //INTEGER64 size;
 PN64_HEADER hh;
-	length = u_lseek64(p->fid,0,SEEK_END);
+	length = fac_lseek(&p->fac,0,SEEK_END);
 	fofs = 0;
 	for ( ; ; ) {
 		if ( fofs == 0 )
@@ -1060,8 +1066,8 @@ PN64_HEADER hh;
 			str = "broken last segment";
 			goto err;
 		}
-		u_lseek64(p->fid,fofs,SEEK_SET);
-		er = u_read(p->fid,&h,sizeof(PN_HEADER));
+		fac_lseek(&p->fac,fofs,SEEK_SET);
+		er = fac_read(&p->fac,&h,sizeof(PN_HEADER));
 		if ( er < sizeof(PN_HEADER) ) {
 			perror("cannot read");
 			er_panic("check_filespace(1)");
@@ -1070,7 +1076,7 @@ PN64_HEADER hh;
 		change_endian(s16);
 		switch ( s16 ) {
 		case PHYS_SIZE32:
-			er = u_read(p->fid,&h.h16+1,
+			er = fac_read(&p->fac,&h.h16+1,
 				sizeof(_PN32_HEADER)
 				- sizeof(PN_HEADER));
 			if ( er < sizeof(_PN32_HEADER)
@@ -1087,7 +1093,7 @@ PN64_HEADER hh;
 			hh.size = h.h32.size;
 			break;
 		case PHYS_SIZE64:
-			er = u_read(p->fid,&h.h16+1,
+			er = fac_read(&p->fac,&h.h16+1,
 				sizeof(_PN64_HEADER)
 				- sizeof(PN_HEADER));
 			if ( er < sizeof(_PN64_HEADER)
@@ -1141,16 +1147,16 @@ char * ptr;
 	if ( debug_pdb64_list(p,DT_DP_SEARCH,fofs,0) < 0 )
 		er_panic("read_filespace");
 	
-	u_lseek64(p->fid,fofs,SEEK_SET);
+	fac_lseek(&p->fac,fofs,SEEK_SET);
 	
-	er = u_read(p->fid,&hh,sizeof(PN_HEADER));
+	er = fac_read(&p->fac,&hh,sizeof(PN_HEADER));
 	if ( er < sizeof(PN_HEADER) )
 		goto err;
 	s16 = hh.h16.size;
 	change_endian(s16);
 	switch ( s16 ) {
 	case PHYS_SIZE32:
-		er = u_read(p->fid,(&hh.h16)+1,
+		er = fac_read(&p->fac,(&hh.h16)+1,
 			sizeof(_PN32_HEADER) - sizeof(PN_HEADER));
 		if ( er < sizeof(_PN32_HEADER) - sizeof(PN_HEADER) )
 			goto err;
@@ -1161,7 +1167,7 @@ char * ptr;
 		_size = size - sizeof(hh.h32) + sizeof(PN64_HEADER);
 		break;
 	case PHYS_SIZE64:
-		er = u_read(p->fid,(&hh.h16)+1,
+		er = fac_read(&p->fac,(&hh.h16)+1,
 			sizeof(_PN64_HEADER) - sizeof(PN_HEADER));
 		if ( er < sizeof(_PN64_HEADER) - sizeof(PN_HEADER) )
 			goto err;
@@ -1191,7 +1197,7 @@ char * ptr;
 		if ( _size > 0x10000 )
 			_s = 0x10000;
 		else	_s = _size;
-		er = u_read(p->fid,ptr,_s);
+		er = fac_read(&p->fac,ptr,_s);
 		if ( er < _s ) {
 			d_f_ree(ah);
 			goto err;
@@ -1218,15 +1224,15 @@ unsigned short s16;
 	if ( debug_pdb64_list(p,DT_DP_SEARCH,fofs,0) < 0 )
 		er_panic("read_filespace");
 	
-	u_lseek64(p->fid,fofs,SEEK_SET);
-	er = u_read(p->fid,&hh,sizeof(PN_HEADER));
+	fac_lseek(&p->fac,fofs,SEEK_SET);
+	er = fac_read(&p->fac,&hh,sizeof(PN_HEADER));
 	if ( er < sizeof(PN_HEADER) )
 		goto err;
 	s16 = hh.h16.size;
 	change_endian(s16);
 	switch ( s16 ) {
 	case PHYS_SIZE32:
-		er = u_read(p->fid,(&hh.h16)+1,
+		er = fac_read(&p->fac,(&hh.h16)+1,
 			sizeof(_PN32_HEADER) - sizeof(PN_HEADER));
 		if ( er < sizeof(_PN32_HEADER) - sizeof(PN_HEADER) )
 			goto err;
@@ -1237,7 +1243,7 @@ unsigned short s16;
 		_size = size - sizeof(hh.h32) + sizeof(PN64_HEADER);
 		break;
 	case PHYS_SIZE64:
-		er = u_read(p->fid,(&hh.h16)+1,
+		er = fac_read(&p->fac,(&hh.h16)+1,
 			sizeof(_PN64_HEADER) - sizeof(PN_HEADER));
 		if ( er < sizeof(_PN64_HEADER) - sizeof(PN_HEADER) )
 			goto err;
@@ -1272,7 +1278,7 @@ U_INTEGER64 size;
 	a = h;
 	size = a->h.size;
 	change_endian(size);
-	u_lseek64(p->fid,a->a.fofs,SEEK_SET);
+	fac_lseek(&p->fac,a->a.fofs,SEEK_SET);
 	return fa_write64_header(p,&a->h,size);
 }
 
@@ -1289,15 +1295,15 @@ unsigned short s16;
 	if ( debug_pdb64_list(p,DT_DP_SEARCH,fofs,0) < 0 )
 		er_panic("read_filespace");
 	
-	u_lseek64(p->fid,fofs,SEEK_SET);
-	er = u_read(p->fid,&hh,sizeof(PN_HEADER));
+	fac_lseek(&p->fac,fofs,SEEK_SET);
+	er = fac_read(&p->fac,&hh,sizeof(PN_HEADER));
 	if ( er < sizeof(PN_HEADER) )
 		goto err;
 	s16 = hh.h16.size;
 	change_endian(s16);
 	switch ( s16 ) {
 	case PHYS_SIZE32:
-		er = u_read(p->fid,(&hh.h16)+1,
+		er = fac_read(&p->fac,(&hh.h16)+1,
 			sizeof(_PN32_HEADER) - sizeof(PN_HEADER));
 		if ( er < sizeof(_PN32_HEADER) - sizeof(PN_HEADER) )
 			goto err;
@@ -1308,7 +1314,7 @@ unsigned short s16;
 		_size = size - sizeof(hh.h32) + sizeof(PN64_HEADER);
 		break;
 	case PHYS_SIZE64:
-		er = u_read(p->fid,(&hh.h16)+1,
+		er = fac_read(&p->fac,(&hh.h16)+1,
 			sizeof(_PN64_HEADER) - sizeof(PN_HEADER));
 		if ( er < sizeof(_PN64_HEADER) - sizeof(PN_HEADER) )
 			goto err;
@@ -1349,8 +1355,8 @@ int err_code;
 	if ( debug_pdb64_list(p,DT_DP_SEARCH,fofs,0) < 0 )
 		er_panic("read_filespace");
 	
-	u_lseek64(p->fid,fofs,SEEK_SET);
-	er = u_read(p->fid,&hh,sizeof(PN_HEADER));
+	fac_lseek(&p->fac,fofs,SEEK_SET);
+	er = fac_read(&p->fac,&hh,sizeof(PN_HEADER));
 	err_code = 1;
 	if ( er < sizeof(PN_HEADER) )
 		goto err;
@@ -1358,7 +1364,7 @@ int err_code;
 	change_endian(s16);
 	switch ( s16 ) {
 	case PHYS_SIZE32:
-		er = u_read(p->fid,(&hh.h16)+1,
+		er = fac_read(&p->fac,(&hh.h16)+1,
 			sizeof(_PN32_HEADER) - sizeof(PN_HEADER));
 		err_code = 2;
 		if ( er < sizeof(_PN32_HEADER) - sizeof(PN_HEADER) )
@@ -1372,7 +1378,7 @@ int err_code;
 		_size = size - sizeof(hh.h32) + sizeof(PN64_HEADER);
 		break;
 	case PHYS_SIZE64:
-		er = u_read(p->fid,(&hh.h16)+1,
+		er = fac_read(&p->fac,(&hh.h16)+1,
 			sizeof(_PN64_HEADER) - sizeof(PN_HEADER));
 		err_code = 3;
 		if ( er < sizeof(_PN64_HEADER) - sizeof(PN_HEADER) )
@@ -1415,7 +1421,7 @@ int err_code;
 		{
 			int debug1;
 			debug1=errno;
-			er = u_read(p->fid,ptr,_s);
+			er = fac_read(&p->fac,ptr,_s);
 			if ( er < _s ) {
 				int en2;
 				en2=errno;
@@ -1443,7 +1449,7 @@ PN64_HEADER * a;
 	a = h;
 	size = a->size;
 	change_endian(size);
-	u_lseek64(p->fid,fofs,SEEK_SET);
+	fac_lseek(&p->fac,fofs,SEEK_SET);
 	return fa_write64_header(p,h,size);
 }
 
@@ -1500,4 +1506,9 @@ limit = 10000000;
 }
 
 
+void
+truncate_filespace64(PDB64 * p)
+{
+	fac_truncate(&p->fac,0);
+}
 
