@@ -1002,6 +1002,7 @@ ss_printf("close_matirx ret_id = %i %i\n",ret_id,matrix_que.total_cnt);
 		d_f_ree(m->filename);
 	if ( m->key )
 		d_f_ree(m->key);
+	d_f_ree(m);
 }
 
 void
@@ -1782,6 +1783,8 @@ see_access_list(t->access_list);
 	default:
 		er_panic("cal");
 	}
+	if ( m->flags & MXF_ERR_DETECTED )
+		goto end;
 	if ( t->access_target_id < 0 ) {
 		t->cal_target = 0;
 		for ( ; get_type(t->cal_target) == XLT_NULL ; ) {
@@ -1815,10 +1818,8 @@ see_access_list(t->access_list);
 	unlock_task(matrix_lock,"matrix_exec_cal");
 	for( ; get_type(t->cal_target) == XLT_PAIR ; ) {
 		lock_task(matrix_lock);
-/*
-		if ( t->process_node->matrix->mode == MM_CLOSE )
+		if ( m->flags & MXF_ERR_DETECTED )
 			goto end;
-*/
 		unlock_task(matrix_lock,"matrix_exec_cal");
 		gc_push(0,0,"matrix_exec_cal");
 
@@ -2436,6 +2437,11 @@ unsigned int key;
 	default:
 		er_panic("_get_matrix_node");
 	}
+	if ( m->flags & MXF_ERR_DETECTED ) {
+		if ( errp )
+			*errp = ME_MATRIX_ERR;
+		return 0;
+	}
 	m->access_cnt |= 1;
 	_delete_matrix_ring(m);
 	_insert_matrix_ring(m);
@@ -2818,6 +2824,8 @@ INTEGER64 lev,target;
 int er_f;
 	lev = dim_code[0];
 	ret = 0;
+	if ( m->flags & MXF_ERR_DETECTED )
+		goto end;
 	if ( access == GN_STEP_NODE_CREATE ) {
 		dim_code[0] = m->total_levels-1;
 		target = m->total_levels-1;
@@ -2920,6 +2928,11 @@ MATRIX_NODE * n;
 int err;
 void * ret;
 
+	if ( m->flags & MXF_ERR_DETECTED ) {
+		if ( errp )
+			*errp = ME_MATRIX_ERR;
+		return 0;
+	}
 	if ( ch < 0 || ch >= m->p.channel_nos ) {
 		if ( errp )
 			*errp = ME_INDEX_OVER;
@@ -3123,6 +3136,10 @@ XL_SEXP * ret;
 char * err_msg;
 
 	err_msg =0;
+	if ( n->matrix->flags & MXF_ERR_DETECTED ) {
+		err_msg = "blocken matrix";
+		goto no_obj;
+	}
 	if  ( n->channel == 0 ) {
 		err_msg = "no data channel";
 		goto no_obj;
@@ -3297,6 +3314,8 @@ MATRIX_DATA_TYPE * md;
 int cf;
 MATRIX_SEXP * s1, * s2;
 	m = n->matrix;
+	if ( m->flags & MXF_ERR_DETECTED )
+		return -1;
 	if ( ch < 0 || ch >= m->p.channel_nos )
 		return -1;
 	if ( n->channel == 0 ) {
@@ -4218,7 +4237,9 @@ MATRIX * m;
 INTEGER64 channel_no,data_size;
 MATRIX_DATA_TYPE * tp;
 int er_data;
+int ret;
 
+	ret = 0;
 	if ( get_type(s) != XLT_RAW )
 		return -1;
 	lock_task(matrix_lock);
@@ -4263,6 +4284,11 @@ int er_data;
 		ptr = get_uncompressed_code64(&data_size,ptr);
 		if ( er_data == 0 ) {
 			tp = m->channel_info[channel_no].data_type;
+			if ( tp == 0 ) {
+				n->matrix->flags |= MXF_ERR_DETECTED;
+				ret = -1;
+				goto err_end;
+			}
 			if ( n->channel[channel_no].data ) {
 				(*tp->free_data)(tp,n->channel[channel_no].data);
 			}
@@ -4271,10 +4297,11 @@ int er_data;
 		}
 		ptr += data_size;
 	}
+err_end:
 	_matrix_node_channel_unlock(n,0);
 	_unlock_node(n,0);
 	unlock_task(matrix_lock,"");
-	return 0;
+	return ret;
 }
 
 XL_SEXP *
