@@ -169,6 +169,7 @@ int ef;
 		n = get_matrix_node_wait(&err,c->m,src,
 					c->gn_wait);
 		if ( err ) {
+//ss_printf("err=%i\n",err);
 			return -1;
 		}
 
@@ -213,10 +214,11 @@ mx_cache_boundary(MX_CACHE * c,int * ix)
 {
 int i,j;
 MATRIX_DH_SET * dsp;
+
 	for ( i = 0 ; i < c->ds_len ; i ++ ) {
 		dsp = &c->ds[i];
 		if ( dsp->ix == 0 )
-			return 0;
+			continue;
 		if ( dsp->hd == 0 )
 			continue;
 		for ( j = 0 ; j < c->m->p.dim ; j ++ ) {
@@ -670,3 +672,4 @@ err:
 }
 
 
+
