@@ -83,6 +83,8 @@ int mx_open_files_cnt;
 extern MATRIX * matrices_root;
 int call_mx_file_gc;
 void test_matrix(MATRIX * m);
+void
+save_cal_check(MATRIX * m);
 
 
 void
@@ -516,7 +518,7 @@ MX_FILE_WORK * w;
 	change_endian(sh.cache_count);
 	change_endian(sh.flags);
 
-	write_filespace64f(w->pdb_fd,w->subheader_fofs,&sh);
+	l_write_filespace64f(w->pdb_fd,w->subheader_fofs,&sh);
 
 	if ( sfa == 0 )
 		finish_file_access(m,0);
@@ -535,7 +537,7 @@ MX_FILE_WORK * w;
 			return -1;
 
 	w = m->file_work;
-	sh = read_filespace64f(w->pdb_fd,w->subheader_fofs);
+	sh = l_read_filespace64f(w->pdb_fd,w->subheader_fofs);
 	if ( sh == 0 ) {
 		finish_file_access(m,0);
 		return -1;
@@ -553,6 +555,7 @@ MX_FILE_WORK * w;
 	change_endian(sh->flags);
 
 	w->subheader = *sh;
+
 /*
 	w->node_root = l_get_root64(&w->favt64_err,
 		w->pdb_fd,
@@ -632,7 +635,7 @@ MATRIX_DATA_TYPE * tp;
 	setup_recordlist64_size(rl);
 	h = rl->data;
 
-	w->header_fofs = alloc_filespace64(w->pdb_fd,rl->data);
+	w->header_fofs = l_alloc_filespace64(w->pdb_fd,rl->data);
 
 	for ( i = 0 ; i < m->p.channel_nos ; i ++ ) {
 		ci = &m->channel_info[i];
@@ -660,7 +663,7 @@ MATRIX_DATA_TYPE * tp;
 	w->subheader.h.size = sizeof(w->subheader);
 	sh = w->subheader.h;
 //	change_endian_header64(&sh);
-	h->subheader_fofs = w->subheader_fofs = alloc_filespace64(w->pdb_fd,&sh);
+	h->subheader_fofs = w->subheader_fofs = l_alloc_filespace64(w->pdb_fd,&sh);
 	w->subheader.cache_next =
 		w->subheader.cache_prev = 
 		w->subheader_fofs;
@@ -679,7 +682,7 @@ MATRIX_DATA_TYPE * tp;
 	change_endian(h->modify_time);
 	change_endian(h->subheader_fofs);
 
-	write_filespace64f(w->pdb_fd,w->header_fofs,rl->data);
+	l_write_filespace64f(w->pdb_fd,w->header_fofs,rl->data);
 
 	free_recordlist64(rl);
 
@@ -837,7 +840,7 @@ er_panic("load");
 					ch.default_data_fofs,
 					tp);
 			if ( info.default_data == 0 ) {
-				free_filespace64(w->pdb_fd,
+				l_free_filespace64(w->pdb_fd,
 					ch.default_data_fofs);
 				info.default_data_fofs = 0;
 			}
@@ -891,10 +894,10 @@ MX_FILE_WORK * w;
 
 	if( fofs == 0 ) {
 	alloc:
-		fofs = alloc_filespace64(w->pdb_fd,rl->data);
+		fofs = l_alloc_filespace64(w->pdb_fd,rl->data);
 	}
 	else {
-		h = read_filespace64f_header(w->pdb_fd,fofs);
+		h = l_read_filespace64f_header(w->pdb_fd,fofs);
 		if ( h == 0 ) {
 			goto alloc;
 		}
@@ -903,13 +906,13 @@ MX_FILE_WORK * w;
 			h2 = (PN64_HEADER*)rl->data;
 			size = h2->size;
 			if ( h->size != size ) {
-				free_filespace64(w->pdb_fd,fofs);
+				l_free_filespace64(w->pdb_fd,fofs);
 				goto alloc;
 			}
 		}
 	}
 	change_endian_header64(rl->data);
-	write_filespace64f(w->pdb_fd,fofs,rl->data);
+	l_write_filespace64f(w->pdb_fd,fofs,rl->data);
 
 	free_recordlist64(rl);
 	
@@ -954,10 +957,10 @@ PN64_MX_DATA_2 * mx;
 
 	if( fofs == 0 ) {
 	alloc:
-		fofs = alloc_filespace64(w->pdb_fd,rl->data);
+		fofs = l_alloc_filespace64(w->pdb_fd,rl->data);
 	}
 	else {
-		h = read_filespace64f_header(w->pdb_fd,fofs);
+		h = l_read_filespace64f_header(w->pdb_fd,fofs);
 		if ( h == 0 ) {
 			goto alloc;
 		}
@@ -966,13 +969,13 @@ PN64_MX_DATA_2 * mx;
 			h2 = (PN64_HEADER*)rl->data;
 			size = h2->size;
 			if ( h->size != size ) {
-				free_filespace64(w->pdb_fd,fofs);
+				l_free_filespace64(w->pdb_fd,fofs);
 				goto alloc;
 			}
 		}
 	}
 	change_endian_header64(&mx->h);
-	write_filespace64f(w->pdb_fd,fofs,rl->data);
+	l_write_filespace64f(w->pdb_fd,fofs,rl->data);
 
 	free_recordlist64(rl);
 	
@@ -998,7 +1001,7 @@ void * ret;
 		return 0;
 
 	w = m->file_work;
-	_r = read_filespace64f(w->pdb_fd,fofs);
+	_r = l_read_filespace64f(w->pdb_fd,fofs);
 	if ( _r == 0 )
 		return 0;
 
@@ -1057,7 +1060,7 @@ unsigned char * ptr;
 		return -1;
 
 	w = m->file_work;
-	n = read_filespace64f(w->pdb_fd,fofs);
+	n = l_read_filespace64f(w->pdb_fd,fofs);
 	if ( n == 0 ) {
 		finish_file_access(m,0);
 ss_printf("load_raw_matrix_node1\n");
@@ -1139,7 +1142,7 @@ MX_FILE_WORK * w;
 	}
 
 	w = m->file_work;
-	write_filespace64f(w->pdb_fd,fofs,org);
+	l_write_filespace64f(w->pdb_fd,fofs,org);
 
 	finish_file_access(m,0);
 	return 0;
@@ -1155,7 +1158,7 @@ MX_FILE_WORK * w;
 	if ( start_file_access(m) < 0 )
 		return -1;
 	w = m->file_work;
-	cr = read_filespace64f(
+	cr = l_read_filespace64f(
 			w->pdb_fd,cr_fofs);
 	change_endian_header64(&cr->h);
 	if ( cr->h.type != PNT_MX_CACHE_RING )
@@ -1172,11 +1175,11 @@ MX_FILE_WORK * w;
 		w->subheader.cache_next = cr->next;
 	}
 	else {
-		cr_buf = read_filespace64f(
+		cr_buf = l_read_filespace64f(
 				w->pdb_fd,cr->prev);
 		cr_buf->prev = cr->prev;
 		change_endian(cr_buf->prev);
-		write_filespace64f(
+		l_write_filespace64f(
 			w->pdb_fd,cr->prev,cr_buf);
 		d_f_ree(cr_buf);
 	}
@@ -1184,7 +1187,7 @@ MX_FILE_WORK * w;
 		w->subheader.cache_prev = cr->prev;
 	}
 	else {
-		cr_buf = read_filespace64f(
+		cr_buf = l_read_filespace64f(
 				w->pdb_fd,cr->next);
 		cr_buf->next = cr->next;
 		change_endian(cr_buf->next);
@@ -1203,14 +1206,14 @@ setup_mx_cache_ring(MATRIX * m,INTEGER64 ring,INTEGER64 node)
 PN64_MX_CACHE_RING * cr;
 MX_FILE_WORK * w;
 	w = m->file_work;
-	cr = read_filespace64f(w->pdb_fd,ring);
+	cr = l_read_filespace64f(w->pdb_fd,ring);
 	change_endian_header64(&cr->h);
 	if ( cr->h.type != PNT_MX_CACHE_RING )
 		er_panic("delete_mx_cache_ring");
 	change_endian_header64(&cr->h);
 	cr->node = node;
 	change_endian(cr->node);
-	write_filespace64f(w->pdb_fd,ring,cr);
+	l_write_filespace64f(w->pdb_fd,ring,cr);
 	d_f_ree(cr);
 }
 
@@ -1232,20 +1235,20 @@ FAVT64_ROOT * rt;
 
 	dim_code = d_alloc(sizeof(MX_DIM_CODE)+sizeof(INTEGER64)*(m->p.dim+1));
 	load_raw_matrix_node(&mx_n,0,(INTEGER64*)(dim_code+1),0,m,dr.node);
-	free_filespace64(w->pdb_fd,dr.node);
-	free_filespace64(w->pdb_fd,cr_fofs);
+	l_free_filespace64(w->pdb_fd,dr.node);
+	l_free_filespace64(w->pdb_fd,cr_fofs);
 	for ( ch = mx_n.channel_fofs ; ch ; ) {
-		d2 = read_filespace64f(w->pdb_fd,ch);
+		d2 = l_read_filespace64f(w->pdb_fd,ch);
 		if ( d2 == 0 )
 			break;
 		change_endian(d2->next_fofs);
 		next = d2->next_fofs;
 		d_f_ree(d2);
-		free_filespace64(w->pdb_fd,ch);
+		l_free_filespace64(w->pdb_fd,ch);
 		ch = next;
 	}
 	if ( mx_n.d.p3.f.nlist_dim_addr )
-		free_filespace64(w->pdb_fd,mx_n.d.p3.f.nlist_dim_addr);
+		l_free_filespace64(w->pdb_fd,mx_n.d.p3.f.nlist_dim_addr);
 	rt = l_get_root64(&w->favt64_err,
 		w->pdb_fd,
 		w->subheader.node_root,
@@ -1268,7 +1271,7 @@ PN64_MX_CACHE_RING * cr,* cr_buf;
 		return -1;
 
 	w = m->file_work;
-	cr = read_filespace64f(w->pdb_fd,cr_fofs);
+	cr = l_read_filespace64f(w->pdb_fd,cr_fofs);
 	change_endian_header64(&cr->h);
 	if ( cr->h.type != PNT_MX_CACHE_RING )
 		er_panic("delete_mx_cache_ring");
@@ -1285,11 +1288,11 @@ PN64_MX_CACHE_RING * cr,* cr_buf;
 
 		w->subheader.cache_next = cr_fofs;
 
-		cr_buf = read_filespace64f(
+		cr_buf = l_read_filespace64f(
 				w->pdb_fd,cr->next);
 		cr_buf->prev = cr_fofs;
 		change_endian(cr_buf->prev);
-		write_filespace64f(w->pdb_fd,cr->next,cr_buf);
+		l_write_filespace64f(w->pdb_fd,cr->next,cr_buf);
 		d_f_ree(cr_buf);
 	}
 	w->subheader.cache_count ++;
@@ -1298,7 +1301,7 @@ PN64_MX_CACHE_RING * cr,* cr_buf;
 	change_endian_header64(&cr->h);
 	change_endian(cr->next);
 	change_endian(cr->prev);
-	write_filespace64f(w->pdb_fd,cr_fofs,cr);
+	l_write_filespace64f(w->pdb_fd,cr_fofs,cr);
 	d_f_ree(cr);
 
 	for ( ; w->subheader.cache_count > mx_cache_count_max ; )
@@ -1374,7 +1377,7 @@ MATRIX * m;
 		return -1;
 
 	w = m->file_work;
-	da = read_filespace64f(w->pdb_fd,addr_fofs);
+	da = l_read_filespace64f(w->pdb_fd,addr_fofs);
 	if ( da == 0 ) {
 		finish_file_access(m,0);
 		return -1;
@@ -1579,10 +1582,10 @@ int debug;
 			cr.h.size = sizeof(PN64_MX_CACHE_RING);
 			cr.node = n->fofs;
 			change_endian(cr.node);
-			n->ring_fofs = alloc_filespace64(
+			n->ring_fofs = l_alloc_filespace64(
 				w->pdb_fd,&cr.h);
 			change_endian_header64(&cr.h);
-			write_filespace64f(w->pdb_fd,n->ring_fofs,&cr);
+			l_write_filespace64f(w->pdb_fd,n->ring_fofs,&cr);
 			insert_mx_cache_ring(m,n->ring_fofs);
 		}
 
@@ -1609,14 +1612,14 @@ int debug;
 		da->h.size = ptr - (unsigned char*)da;
 		if ( n->dim_addr_size != da->h.size &&
 				n->dim_addr_fofs ) {
-			free_filespace64(w->pdb_fd,n->dim_addr_fofs);
+			l_free_filespace64(w->pdb_fd,n->dim_addr_fofs);
 			n->dim_addr_fofs = 0;
 		}
 		if ( n->dim_addr_fofs == 0 )
 			n->dim_addr_fofs =
-				alloc_filespace64(w->pdb_fd,&da->h);
+				l_alloc_filespace64(w->pdb_fd,&da->h);
 		change_endian_header64(&da->h);
-		write_filespace64f(w->pdb_fd,
+		l_write_filespace64f(w->pdb_fd,
 			n->dim_addr_fofs,da);
 		d_f_ree(da);
 
@@ -1637,7 +1640,7 @@ int debug;
 debug = 0;
 	if ( n->fofs == 0 ) {
 debug = 1;
-		n->fofs = alloc_filespace64(w->pdb_fd,rl->data);
+		n->fofs = l_alloc_filespace64(w->pdb_fd,rl->data);
 		if ( n->ring_fofs )
 			setup_mx_cache_ring(m,n->ring_fofs,n->fofs);
 
@@ -1672,7 +1675,7 @@ debug = 1;
 
 //ss_printf("NODE SIZE " I64_FORMAT "\n",h->size);
 	change_endian_header64(h);
-	write_filespace64f(w->pdb_fd,n->fofs,rl->data);
+	l_write_filespace64f(w->pdb_fd,n->fofs,rl->data);
 	wakeup_task((int)n);
 
 	finish_file_access(m,0);
@@ -1691,6 +1694,10 @@ MX_FILE_WORK * w;
 	w = m->file_work;
 	w->subheader.flags &= ~MXF_WRITE;
 	save_subheader(m,1);
+ss_printf("CLOSE DETECT FLAG %x\n",m->flags);
+	if ( m->flags & MXF_ERR_DETECTED ) {
+		l_truncate_filespace64(w->pdb_fd);
+	}
 	l_close_filespace64(w->pdb_fd);
 	mx_open_files_cnt --;
 	call_unlock(w->lock_d);
@@ -1932,7 +1939,7 @@ STREAM * st;
 				cals[i] = 0;
 				continue;
 			}
-			c = read_filespace64f(w->pdb_fd,fofs);
+			c = l_read_filespace64f(w->pdb_fd,fofs);
 			if ( c == 0 ) {
 				cals[i] = 0;
 				continue;
@@ -1951,6 +1958,41 @@ STREAM * st;
 }
 
 
+void
+save_cal_check(MATRIX * m)
+{
+int i,j;
+int mode1,mode2;
+MX_FILE_WORK * w;
+
+ss_printf("SCC\n");
+	w = m->file_work;
+	for ( mode1 = 0 ; mode1 < MI_MODE_MAX ; mode1 ++ ) {
+		for ( i = 0 ; i < MI_MAX ; i ++ ) {
+			if ( w->subheader.mode_cal_fofs[mode1][i] == 0 )
+				continue;
+ss_printf("SCC %i-%i -- %lli\n",mode1,i, w->subheader.mode_cal_fofs[mode1][i]);
+			mode2 = mode1;
+			for ( j = i+1 ; j < MI_MAX ; j ++ ) {
+				if ( w->subheader.mode_cal_fofs[mode2][j]
+					== w->subheader.mode_cal_fofs[mode1][i] ) {
+					sleep_sec(2);
+					er_panic("save_cal_Check");
+				}
+			}
+			for ( mode2 = mode1+1 ; mode2 < MI_MODE_MAX ; mode2 ++ ) {
+				for ( j = 0 ; j < MI_MAX ; j ++ ) {
+					if ( w->subheader.mode_cal_fofs[mode2][j]
+						== w->subheader.mode_cal_fofs[mode1][i] ) {
+						sleep_sec(2);
+						er_panic("save_cal_Check");
+					}
+				}
+			}
+		}
+	}
+}
+
 int
 save_mode_cal(MATRIX * m,int mode,char *** cals,int * pri)
 {
@@ -1989,23 +2031,23 @@ int f;
 				if ( fofs == 0 ) {
 				new_alloc:
 					f = 1;
-					fofs = alloc_filespace64(w->pdb_fd,rl->data);
+					fofs = l_alloc_filespace64(w->pdb_fd,rl->data);
 				}
 				else {
-					c = read_filespace64f(w->pdb_fd,fofs);
+					c = l_read_filespace64f(w->pdb_fd,fofs);
 					if ( c == 0 ) {
 						goto new_alloc;
 					}
 					change_endian_header64(&c->h);
 					if ( h->h.size != c->h.size ) {
 						d_f_ree(c);
-						free_filespace64(w->pdb_fd,fofs);
+						l_free_filespace64(w->pdb_fd,fofs);
 						goto new_alloc;
 					}
 					d_f_ree(c);
 				}
 				change_endian_header64(rl->data);
-				write_filespace64f(w->pdb_fd,fofs,rl->data);
+				l_write_filespace64f(w->pdb_fd,fofs,rl->data);
 				w->subheader.mode_cal_fofs[mode][i] = fofs;
 				
 				free_recordlist64(rl);
@@ -2013,13 +2055,14 @@ int f;
 			else {
 				fofs = w->subheader.mode_cal_fofs[mode][i];
 				if ( fofs ) {
-					free_filespace64(w->pdb_fd,fofs);
+					l_free_filespace64(w->pdb_fd,fofs);
 					w->subheader.mode_cal_fofs[mode][i] = 0;
 					f = 1;
 				}
 			}
 		}
 	}
+
 	if ( f )
 		save_subheader(m,0);
 	finish_file_access(m,0);
