@@ -282,7 +282,7 @@ _s_printf_1(STREAM *s,int * erp, char * fmt, va_list p){
 		
 		for( ; *str; str++){
 			if ( s->h.cr_mode &&
-				(*str == '\n' || *str == '\r') ) {
+				(*str == '\n' || *str == '\r' || *str == C_CANCEL ) ) {
 				s->h.cr_cnt ++;
 			}
 			if ( len-ptr < cm->max_ret ) {
@@ -399,11 +399,19 @@ flush_spsat(S_PRINTF_SAT * sat,int * erp)
 
 	int pri;
 
+	if ( sat->err < 0 ) {
+		if ( erp )
+			*erp = ESYS_INVAL;
+		return -1;
+	}
+
 	pri = push_pri(PRI_NETWORK);
 	lock_task(stream_lock);
 	if ( sat->st == 0 ) {
 		if ( erp )
 			*erp = ESYS_INVAL;
+		sat->err = -1;
+		sat->str_buf_ptr = 0;
 		unlock_task(stream_lock,"s_write");
 		change_pri(0,pri);
 		return -1;
@@ -411,6 +419,8 @@ flush_spsat(S_PRINTF_SAT * sat,int * erp)
 	if ( sat->st->h.tbl == 0 ) {
 		if ( erp )
 			*erp = ESYS_INVAL;
+		sat->err = -1;
+		sat->str_buf_ptr = 0;
 		unlock_task(stream_lock,"s_write");
 		change_pri(0,pri);
 		return -1;
@@ -430,7 +440,7 @@ flush_spsat(S_PRINTF_SAT * sat,int * erp)
 	
 	for( ; size ; str++, size -- ){
 		if ( sat->st->h.cr_mode &&
-			(*str == '\n' || *str == '\r') ) {
+			(*str == '\n' || *str == '\r' || *str == C_CANCEL ) ) {
 			sat->st->h.cr_cnt ++;
 		}
 		if ( len-ptr < cm->max_ret ) {
@@ -500,6 +510,7 @@ flush_spsat(S_PRINTF_SAT * sat,int * erp)
 				}
 				if ( erp )
 					*erp = errno;
+				sat->err = -1;
 				break;
 			}
 			else calc_checksum(sat->st,pp,er,2);
@@ -514,7 +525,7 @@ flush_spsat(S_PRINTF_SAT * sat,int * erp)
 		s_delete_thread(sat->st,&t);
 	}
 	sat->str_buf_ptr = 0;
-	return 0;
+	return sat->err;
 }
 
 
@@ -539,10 +550,13 @@ int
 write_spsat(S_PRINTF_SAT * sat,L_CHAR * data,int len)
 {
 int er;
+	if ( sat->err < 0 )
+		return -1;
 	for ( ; len ; ) {
 		if ( sat->str_buf_ptr == S_PRINTF_OUTPUT_BUF_LEN ) {
 			er = 0;
-			flush_spsat(sat,&er);
+			if ( flush_spsat(sat,&er) < 0 )
+				return -1;
 			if ( er < 0 )
 				return -1;
 		}
@@ -553,24 +567,30 @@ int er;
 	return 0;
 }
 
-void
+int
 new_spsat(S_PRINTF_SAT * sat,STREAM * st)
 {
+	if ( st == 0 )
+		return -1;
 	memset(sat,0,sizeof(*sat));
 	sat->st = st;
 	sat->str_buf = d_alloc(S_PRINTF_OUTPUT_BUF_LEN*sizeof(L_CHAR));
+	return 0;
 }
 
-void
+int
 finish_spsat(S_PRINTF_SAT * sat,int * erp)
 {
 int i;
+int ret;
+	ret = 0;
 	if ( sat->str_buf_ptr )
-		flush_spsat(sat,erp);
+		ret = flush_spsat(sat,erp);
 	d_f_ree(sat->str_buf);
 	for ( i = 0 ; i < SPSAT_MAX ; i ++ )
 		if ( sat->alc[i] )
 			d_f_ree(sat->alc[i]);
+	return ret;
 }
 
 
@@ -707,7 +727,11 @@ _s_printf_2(S_PRINTF_SAT * sat,int * erp, char * fmt, va_list p){
 
 	out.mem[out.len] = 0;
 	
-	write_spsat(sat,out.mem,out.len);
+	if ( write_spsat(sat,out.mem,out.len) < 0 ) {
+		if ( erp )
+			*erp = ESYS_INVAL;
+		return -1;
+	}
 end:	
 
 	return out.len;
@@ -716,6 +740,7 @@ end:
 int 
 _s_printf(STREAM *s,int * erp, char * fmt, va_list p,S_PRINTF_SAT * sat)
 {
+
 	if ( sat )
 		return _s_printf_2(sat,erp,fmt,p);
 //return _s_printf_1(sat->st,erp,fmt,p);
