@@ -372,7 +372,7 @@ L_CHAR * str;
 		_ret = code_convert_with_combine(str,l_strlen(str),
 				sjis_cm.main_code,CBF_SRC_PLANE|CBF_DST_PLANE);
 */
-		sscanf(n_string(&sjis_cm,str),"%f",tp);
+		sscanf(n_string(&sjis_cm,str),REAL1_F,tp);
 		_set_dirty_flag();
 		return 1;
 	}
