@@ -105,7 +105,7 @@ int len;
 	d = (char*)d_alloc(len = p - str + 1);
 	memcpy(d,str,len);
 	d[p-str] = 0;
-	sscanf(str,"%f",ret);
+	sscanf(str,REAL1_F,ret);
 	d_f_ree(d);
 	if ( *p == 0 )
 		return 0;
@@ -162,12 +162,12 @@ L_CHAR * d;
 	}
 	d = get_sf_attribute(sf,l_string(std_cm,"flame.crd.FlameBaseRotate"));
 	if ( d ) {
-		sscanf(n_string(std_cm,d),"%f",&sts->flame_base_rotate);
+		sscanf(n_string(std_cm,d),REAL1_F,&sts->flame_base_rotate);
 		(*flags) |= VSF_FLAME_C_FLAME_BASE_ROTATE;
 	}
 	d = get_sf_attribute(sf,l_string(std_cm,"flame.crd.FlameBaseResolution"));
 	if ( d ) {
-		sscanf(n_string(std_cm,d),"%f",&sts->flame_base_resolution);
+		sscanf(n_string(std_cm,d),REAL1_F,&sts->flame_base_resolution);
 		(*flags) |= VSF_FLAME_C_FLAME_BASE_RESOLUTION;
 	}
 	d = get_sf_attribute(sf,l_string(std_cm,"flame.crd.Target"));
