@@ -646,7 +646,7 @@ VCustomizedMenuBar::menu_choosed(VMenuItem *item)
 	VEditable *focused = VEditable::get_focused_object();
 	if ( item == 0 )
 		return false;
-	if ( focused && item->type != VMT_OTHER && item->type != VMT_SEPARATOR )
+	if ( focused && item->type != VMT_OTHER && item->type != VMT_SEPARATOR && item->type >= 0 )
 		if ( focused->obey_command(item->type) )
 			return true;
 	
