@@ -42,11 +42,23 @@ vobj_VChooseFile(XLISP_ENV *env, XL_SEXP *arg, XLISP_ENV *a, XL_SYM_FIELD *sf)
 	return ret ? n_get_string(ret) : 0;
 }
 
+XL_SEXP *
+vobj_VSaveAsFile(XLISP_ENV *env, XL_SEXP *arg, XLISP_ENV *a, XL_SYM_FIELD *sf)
+{
+	L_CHAR *title = get_sf_attribute(sf, l_string(std_cm, "title"));
+	L_CHAR *def = get_sf_attribute(sf, l_string(std_cm, "default"));
+	
+	char *ret = v_save_as_file(title, def ? n_string(std_cm, def) : 0);
+	return ret ? n_get_string(ret) : 0;
+}
+
 void
 init_VChooseFile(XLISP_ENV *env)
 {
 	set_env(env,l_string(std_cm,"VChooseFile"),
 		get_func_prim((XL_SEXP*(*)())vobj_VChooseFile,FO_NORMAL,0,1,1));
+	set_env(env,l_string(std_cm,"VSaveAsFile"),
+		get_func_prim((XL_SEXP*(*)())vobj_VSaveAsFile,FO_NORMAL,0,1,1));
 }
 
 } // extern "C"
