@@ -17,11 +17,13 @@
 
 #include "v/VMacro.h"
 #include "v/v.h"
+#include "v/vobj_utils.h"
 
 extern "C" {
 
 #include "memory_debug.h"
 #include "xldir.h"
+#include "xlerror.h"
 
 void init_VMacro();
 void init_tempolary_files();
