@@ -25,7 +25,7 @@ XL_SEXP * xl_Cast();
 XL_SEXP * xl_TypeOf();
 
 //char type_table[XLT_MAX][12] = {
-char * type_table[12] = {
+char * type_table[XLT_MAX] = {
 	"null",
 	"error",
 	"list",
@@ -110,7 +110,7 @@ XL_SEXP * _ret;
 			if ( 'a' <= *pp && *pp <= 'z' )
 				*pp = *pp + 'A' - 'a';
 	}
-	_ret = get_symbol(ret);
+	_ret = get_string(ret);
 	d_f_ree(ret);
 	return _ret;
 }
