@@ -37,7 +37,7 @@ int parse_lock_cnt;
 int parse_lock_pri;
 
 
-PRI_CTL parse_lock_ctl = {PRI_NETWORK,0,0};
+PRI_CTL parse_lock_ctl = {PRI_PARSE,0,0};
 
 #define ACTIVE 629
 
