@@ -110,10 +110,10 @@ void empty_reply_task();
 /*
 	new_tick(check_fid_tick,2,2);
 */
-	create_task(empty_reply_task,0,1);
+	create_task((void(*)(TKEY))empty_reply_task,0,1);
 
 	ss_run_tasks = 0;
-	create_task(ss_task,0,1);
+	create_task((void(*)(TKEY))ss_task,0,1);
 }
 
 
@@ -548,7 +548,7 @@ int ret;
 			ss_wait_tasks --;
 			ss_run_tasks ++;
 			if ( ss_wait_tasks <= 0 && ss_run_tasks < 10 )
-				create_task(ss_task,0,1);
+				create_task((void(*)(TKEY))ss_task,0,1);
 		}
 		unlock_task(XoH_lock,"ss_task");
 		ret = new_accept(&new,&ss->key,ss->xli,ss->pl);
