@@ -1580,6 +1580,8 @@ _accept_ptr = _accept_ptr % 1000;
 		unlock_task(xli_lock,"_accept_thread");
 
 		key = accept_connection(xli->inp);
+		if ( key && key->s )
+			s_set_write_abort(key->s,WAT_CLOSE,WAT_DEFAULT_INTERVAL);
 
 		lock_task(xli_lock);
 		xli->accept_lock = 0;
