@@ -1587,7 +1587,7 @@ LOG_NODE_DATA * opt;
 		ln->d.d.u.agent = ll_copy_str(agent_name);
 	else	ln->d.d.u.agent = 0;
 
-	_s_printf(st,0,fmt,p);
+	_s_printf(st,0,fmt,p,0);
 	ln->d.d.data = ll_copy_str(s_get_l_string(st));
 
 	s_close(st);
